Package: regsplice
Version: 1.36.0
Title: L1-regularization based methods for detection of differential
        splicing
Description: Statistical methods for detection of differential splicing 
    (differential exon usage) in RNA-seq and exon microarray data, using 
    L1-regularization (lasso) to improve power.
Authors@R: person("Lukas M.", "Weber", email = "lmweb012@gmail.com", role = c("aut", "cre"))
Author: Lukas M. Weber [aut, cre]
Maintainer: Lukas M. Weber <lmweb012@gmail.com>
URL: https://github.com/lmweber/regsplice
BugReports: https://github.com/lmweber/regsplice/issues
License: MIT + file LICENSE
Imports: glmnet, SummarizedExperiment, S4Vectors, limma, edgeR, stats,
        pbapply, utils, methods
Suggests: testthat, BiocStyle, knitr, rmarkdown
VignetteBuilder: knitr
RoxygenNote: 7.1.0
biocViews: ImmunoOncology, AlternativeSplicing, DifferentialExpression,
        DifferentialSplicing, Sequencing, RNASeq, Microarray,
        ExonArray, ExperimentalDesign, Software
Collate: 'class_RegspliceResults.R' 'class_RegspliceData.R' 'LRTests.R'
        'createDesignMatrix.R' 'filterLowCounts.R' 'filterZeros.R'
        'fitting_functions_multiple_genes.R'
        'fitting_functions_single_gene.R' 'helper_functions.R'
        'initializeResults.R' 'regsplice_wrapper.R'
        'runNormalization.R' 'runVoom.R' 'summaryTable.R'
git_url: https://git.bioconductor.org/packages/regsplice
git_branch: RELEASE_3_22
git_last_commit: d9c8d76
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-29
NeedsCompilation: no
Packaged: 2025-10-30 06:00:59 UTC; biocbuild

# Generated by roxygen2: do not edit by hand

export(LRTests)
export(LR_stats)
export(RegspliceData)
export(RegspliceResults)
export(countsData)
export(createDesignMatrix)
export(df_tests)
export(filterLowCounts)
export(filterZeros)
export(fitFullMultiple)
export(fitNullMultiple)
export(fitRegMultiple)
export(gene_IDs)
export(initializeResults)
export(p_adj)
export(p_vals)
export(regsplice)
export(runNormalization)
export(runVoom)
export(summaryTable)
export(weightsData)
exportClasses(RegspliceData)
exportClasses(RegspliceResults)
exportMethods("[")
exportMethods(LR_stats)
exportMethods(assays)
exportMethods(colData)
exportMethods(countsData)
exportMethods(df_tests)
exportMethods(gene_IDs)
exportMethods(p_adj)
exportMethods(p_vals)
exportMethods(rowData)
exportMethods(weightsData)
importFrom(S4Vectors,DataFrame)
importFrom(S4Vectors,SimpleList)
importFrom(SummarizedExperiment,"assays<-")
importFrom(SummarizedExperiment,"colData<-")
importFrom(SummarizedExperiment,Assays)
importFrom(SummarizedExperiment,SummarizedExperiment)
importFrom(SummarizedExperiment,assays)
importFrom(SummarizedExperiment,colData)
importFrom(SummarizedExperiment,rowData)
importFrom(edgeR,calcNormFactors)
importFrom(glmnet,cv.glmnet)
importFrom(limma,voom)
importFrom(methods,callNextMethod)
importFrom(methods,is)
importFrom(methods,new)
importFrom(pbapply,pblapply)
importFrom(pbapply,pboptions)
importFrom(stats,deviance)
importFrom(stats,glm)
importFrom(stats,model.matrix)
importFrom(stats,p.adjust)
importFrom(stats,pchisq)
importFrom(utils,head)

########################
### Helper functions ###
########################


# Remove genes containing only a single exon bin (since differential splicing requires
# multiple exon bins).
# 
.removeSingleExonGenes <- function(rs_data) {
  
  if (!("RegspliceData" %in% is(rs_data))) stop("'rs_data' must be a 'RegspliceData' object")
  
  gene_IDs_rep <- rowData(rs_data)$gene_IDs
  gene_IDs <- names(table(gene_IDs_rep))
  gene_IDs_singles <- gene_IDs[table(gene_IDs_rep) == 1]
  
  ix_singles <- gene_IDs_rep %in% gene_IDs_singles
  
  message(paste("removed", sum(ix_singles), "remaining single-exon gene(s)"))
  
  suppressMessages(rs_data[!ix_singles, ])
}

#' Create design matrix.
#' 
#' Create a model design matrix for a single gene.
#' 
#' Creates a model design matrix for a single gene in the format required by the 
#' \code{regsplice} model fitting functions. Required inputs are the experimental 
#' conditions (groups) for each sample, and the number of exons in the gene.
#' 
#' The design matrix includes main effect terms for each exon and each sample, and
#' interaction terms between the exons and conditions.
#' 
#' Note that the design matrix does not include main effect terms for the conditions,
#' since these are absorbed into the main effect terms for the samples. In addition, the
#' design matrix does not include an intercept column, since it is simpler to let the
#' model fitting functions add an intercept term later.
#' 
#' The model fitting functions in subsequent steps call this function once for each gene.
#' 
#' @param condition Experimental conditions for each sample (character or numeric vector,
#'   or factor).
#' @param n_exons Number of exons in the gene (integer).
#' 
#' @return Returns a model design matrix for the gene, in the format required by the 
#'   \code{regsplice} model fitting functions.
#' 
#' @seealso \code{\link{fitRegMultiple}} \code{\link{fitNullMultiple}} 
#'   \code{\link{fitFullMultiple}} \code{\link{LRTests}}
#' 
#' @importFrom stats model.matrix
#' 
#' @export
#'   
#' @examples
#' condition <- rep(c(0, 1), each = 3)
#' n_exons <- 10
#' X <- createDesignMatrix(condition, n_exons)
#' 
createDesignMatrix <- function(condition, n_exons) {
  
  n_samples <- length(condition)
  
  Exon <- factor(rep(seq_len(n_exons), times = n_samples))
  Samp <- factor(rep(seq_len(n_samples), each = n_exons))
  Cond <- factor(rep(condition, each = n_exons))
  
  # Build design matrix manually, since creating it with 'model.matrix(~ Exon + Samp + 
  # Exon:Cond)[, -1]' includes an extra (linearly dependent) interaction column for the 
  # first exon (this is due to the absence of the main effect term Cond). Also do not 
  # include an intercept column, since it is simpler to let the model fitting functions 
  # add it back later.
  main_effects <- stats::model.matrix(~ Exon + Samp)[, -1, drop = FALSE]
  int_temp     <- stats::model.matrix(~ Exon + Cond + Exon:Cond)[, -1, drop = FALSE]
  interactions <- int_temp[, grep(":", colnames(int_temp)), drop = FALSE]
  
  X <- cbind(main_effects, interactions)
}

library(regsplice)
context("Design matrices")

test_that("design matrix is generated correctly for saved example", {
  
  condition <- rep(c(0, 1), each = 3)
  n_exons <- 4
  design <- createDesignMatrix(condition = condition, n_exons = n_exons)
  
  file_saved <- system.file("extdata/design_matrix_example.txt", package = "regsplice")
  design_saved <- as.matrix(read.table(file_saved, header = TRUE, check.names = FALSE))
  
  expect_equivalent(design, design_saved)
})

#' @include class_RegspliceData.R class_RegspliceResults.R
NULL




#' Summary table.
#' 
#' Display summary table of results from a \code{regsplice} analysis.
#' 
#' The results of a \code{regsplice} analysis consist of a set of multiple testing 
#' adjusted p-values (Benjamini-Hochberg false discovery rates, FDR) quantifying the 
#' statistical evidence for differential exon usage (DEU) for each gene. Typically, the
#' adjusted p-values are used to rank the genes in the data set according to their
#' evidence for DEU, and an appropriate significance threshold (e.g. FDR < 0.05) can be
#' used to generate a list of genes with statistically significant evidence for DEU.
#' 
#' The main \code{regsplice} functions return results in the form of a 
#' \code{\linkS4class{RegspliceResults}} object, which contains slots for gene names, 
#' fitted model results, raw p-values, multiple testing adjusted p-values 
#' (Benjamini-Hochberg FDR), likelihood ratio (LR) test statistics, and degrees of 
#' freedom of the LR tests. See \code{\linkS4class{RegspliceResults}} and the main 
#' \code{regsplice} wrapper function \code{\link{regsplice}} for details.
#' 
#' This function generates a summary table of the results. The results are displayed as a
#' data frame of the top \code{n} most highly significant genes, ranked according to 
#' either FDR or raw p-values, up to a specified significance threshold (e.g. FDR < 
#' 0.05).
#' 
#' The argument \code{rank_by} controls whether to rank by FDR or raw p-values. The
#' default is to rank by FDR.
#' 
#' To display results for all genes up to the significance threshold, set the argument 
#' \code{n = Inf}. To display results for all genes in the data set, set both \code{n = 
#' Inf} and \code{threshold = 1}.
#' 
#' Previous step: Run \code{regsplice} pipeline with the \code{\link{regsplice}} wrapper
#' function (or individual functions up to \code{\link{LRTests}}).
#' 
#' 
#' @param rs_results \code{\linkS4class{RegspliceResults}} object containing results of a
#'   \code{regsplice} analysis, generated with wrapper function \code{\link{regsplice}} 
#'   (or individual functions up to \code{\link{LRTests}}). See 
#'   \code{\linkS4class{RegspliceResults}} for details.
#' @param n Number of genes to display in summary table. Default is 20. If the total
#'   number of significant genes up to the significance threshold is less than \code{n},
#'   only the significant genes are shown. Set to \code{Inf} to display all significant
#'   genes; or set both \code{n = Inf} and \code{threshold = 1} to display all genes in
#'   the data set.
#' @param threshold Significance threshold (for either FDR or raw p-values, depending on 
#'   choice of argument \code{rank_by}). Default is 0.05. Set to 1 to display all 
#'   \code{n} genes; or set both \code{n = Inf} and \code{threshold = 1} to display all 
#'   genes in the data set.
#' @param rank_by Whether to rank genes by false discovery rate (FDR), raw p-values, or 
#'   no ranking. Choices are \code{"FDR"}, \code{"p-value"}, and \code{"none"}. Default 
#'   is \code{"FDR"}.
#' 
#' 
#' @return Returns a data frame containing results for the top \code{n} most highly 
#'   significant genes, up to the specified significance threshold for the FDR or raw 
#'   p-values.
#' 
#' @seealso \code{\linkS4class{RegspliceResults}} \code{\link{regsplice}}
#' 
#' @importFrom utils head
#' @importFrom methods is
#' 
#' @export
#' 
#' @examples
#' file_counts <- system.file("extdata/vignette_counts.txt", package = "regsplice")
#' data <- read.table(file_counts, header = TRUE, sep = "\t", stringsAsFactors = FALSE)
#' head(data)
#' 
#' counts <- data[, 2:7]
#' tbl_exons <- table(sapply(strsplit(data$exon, ":"), function(s) s[[1]]))
#' gene_IDs <- names(tbl_exons)
#' n_exons <- unname(tbl_exons)
#' condition <- rep(c("untreated", "treated"), each = 3)
#' 
#' rs_data <- RegspliceData(counts, gene_IDs, n_exons, condition)
#' 
#' rs_results <- regsplice(rs_data)
#' 
#' summaryTable(rs_results)
#' summaryTable(rs_results, n = Inf, threshold = 1)
#' 
summaryTable <- function(rs_results, n = 20, threshold = 0.05, 
                         rank_by = c("FDR", "p-value", "none")) {
  
  rank_by <- match.arg(rank_by)
  
  if (!("RegspliceResults" %in% is(rs_results))) {
    stop("'rs_results' must be a 'RegspliceResults' object")
  }
  
  if (rank_by == "FDR") {
    ix <- order(rs_results@p_adj)
  } else if (rank_by == "p-value") {
    ix <- order(rs_results@p_vals)
  } else if (rank_by == "none") {
    ix <- seq_along(rs_results@gene_IDs)
  }
  
  out_display <- data.frame(gene_IDs = rs_results@gene_IDs, 
                            p_vals = rs_results@p_vals, 
                            p_adj = rs_results@p_adj, 
                            LR_stats = rs_results@LR_stats, 
                            df_tests = rs_results@df_tests, 
                            stringsAsFactors = FALSE)
  
  out_ordered <- out_display[ix, ]
  row.names(out_ordered) <- NULL
  
  if (rank_by == "FDR") {
    out_sig <- out_ordered[out_ordered$p_adj <= threshold, ]
  } else if (rank_by == "p-value") {
    out_sig <- out_ordered[out_ordered$p_vals <= threshold, ]
  } else if (rank_by == "none") {
    out_sig <- out_ordered
  }
  
  utils::head(out_sig, n = n)
}

library(regsplice)
context("Summary table")

test_that("summary table function works correctly", {
  
  # generate data
  set.seed(123)
  n_exons <- c(7, 18, 5, 5, 1, 3, 11)
  counts <- matrix(sample(100:200, sum(n_exons) * 6, replace = TRUE), ncol = 6)
  gene_IDs <- paste0("gene", 1:length(n_exons))
  condition <- rep(c(0, 1), each = 3)
  
  rs_data <- RegspliceData(counts, gene_IDs, n_exons, condition)
  
  rs_results <- regsplice(rs_data)
  
  table_default             <- summaryTable(rs_results)
  table_all_up_to_threshold <- summaryTable(rs_results, n = Inf)
  table_all                 <- summaryTable(rs_results, n = Inf, threshold = 1)
  
  table_pval <- summaryTable(rs_results, rank_by = "p-value")
  table_none <- summaryTable(rs_results, rank_by = "none")
  
  table_small_default <- summaryTable(rs_results, n = 3)
  table_small_pval    <- summaryTable(rs_results, n = 3, rank_by = "p-value")
  table_small_none    <- summaryTable(rs_results, n = 3, rank_by = "none")
  
  
  expect_equal(ncol(table_default), 5)
  expect_equal(ncol(table_all_up_to_threshold), 5)
  expect_equal(nrow(table_all), 6)
  expect_equal(ncol(table_all), 5)
  
  expect_equal(ncol(table_pval), 5)
  expect_equal(nrow(table_none), 6)
  expect_equal(ncol(table_none), 5)
  
  expect_equal(ncol(table_small_default), 5)
  expect_equal(ncol(table_small_pval), 5)
  expect_equal(nrow(table_small_none), 3)
  expect_equal(ncol(table_small_none), 5)
})

###########################
### S4 class definition ###
###########################


#' @rdname RegspliceData
#' @export
#' 
setClass("RegspliceData", contains = "SummarizedExperiment")




############################
### Constructor function ###
############################


#' RegspliceData objects.
#' 
#' \code{RegspliceData} objects contain data in the format required by functions in the 
#' \code{regsplice} analysis pipeline.
#' 
#' The \code{RegspliceData} format is based on the 
#' \code{\link[SummarizedExperiment]{SummarizedExperiment}} container. Initially, objects
#' contain raw data along with meta-data for rows (genes and exons) and columns 
#' (biological samples). During subsequent steps in the \code{regsplice} analysis 
#' pipeline, the data values are modified, and additional data and meta-data are added to
#' the object. Final results are stored in a \code{\linkS4class{RegspliceResults}} 
#' object.
#' 
#' \code{RegspliceData} objects are created with the constructor function 
#' \code{RegspliceData()}.
#' 
#' Required inputs for the constructor function are \code{counts} (matrix or data frame
#' of RNA-seq read counts or exon microarray intensities), \code{gene_IDs} (vector of
#' gene IDs), \code{n_exons} (vector of exon lengths, i.e. number of exon bins per gene),
#' and \code{condition} (vector of experimental conditions for each biological sample).
#' 
#' Alternatively, the inputs can be provided as a \code{SummarizedExperiment} object, 
#' which will be parsed to extract each of these components. This may be useful when 
#' running \code{regsplice} as part of a pipeline together with other packages.
#' 
#' See the vignette for an example showing how to construct \code{gene_IDs} and 
#' \code{n_exons} from a column of gene:exon IDs.
#' 
#' Exon microarray intensities should be log2-transformed, which is usually done during 
#' pre-processing of microarray data. (RNA-seq counts will be transformed automatically
#' during the \code{regsplice} analysis pipeline; see \code{\link{runVoom}}.)
#' 
#' After creating a \code{RegspliceData} object, the wrapper function 
#' \code{\link{regsplice}} can be used to run the analysis pipeline with a single
#' command. Alternatively, you can run the individual functions for each step in the
#' pipeline, beginning with \code{\link{filterZeros}} (see vignette for details).
#' 
#' 
#' @param counts RNA-seq read counts or exon microarray intensities (matrix or data 
#'   frame). Rows are exons, and columns are biological samples. Alternatively,
#'   \code{counts} also accepts a \code{SummarizedExperiment} input object containing all
#'   required input data, which may be useful when running \code{regsplice} as part of a
#'   pipeline with other packages.
#' @param gene_IDs Vector of gene IDs (character vector). Length is equal to the number 
#'   of genes.
#' @param n_exons Vector of exon lengths (numeric vector of integers), i.e. the number of
#'   exon bins per gene. Length is equal to the number of genes.
#' @param condition Experimental condition for each biological sample (character or 
#'   numeric vector, or factor).
#' @param x \code{RegspliceData} object (for accessor or subsetting functions).
#' @param i Gene names (character vector) or row numbers (numeric vector) for subsetting 
#'   genes or exons. Note that when subsetting whole genes, gene names (character vector)
#'   should be provided instead of row numbers, to avoid possible errors due to selecting
#'   incorrect row numbers. Row numbers may be provided to subset individual exons.
#' @param j Column numbers (numeric vector) for subsetting biological samples.
#' @param ... Additional arguments for replacement with \code{`[<-`}.
#' @param value Value for replacement with \code{`[<-`}.
#' @param withDimnames See \code{SummarizedExperiment::assays()}.
#' 
#' 
#' @field counts Matrix of RNA-seq read counts or exon microarray intensities. Rows are 
#'   exons, and columns are biological samples.
#' @field weights (Optional) Matrix of observation-level weights. Rows are exons, and 
#'   columns are biological samples. Created by the \code{\link{runVoom}} function.
#' @field rowData \code{DataFrame} of row meta-data. This should contain two columns: 
#'   \code{gene_IDs} and \code{exon_IDs}, which are created by the \code{RegspliceData}
#'   constructor function.
#' @field colData \code{DataFrame} of column meta-data. This contains the experimental 
#'   condition and (optionally) normalization factors for each biological sample. 
#'   Normalization factors are created by the \code{\link{runVoom}} function.
#' 
#' 
#' @section Accessor functions:
#' 
#' \itemize{
#' \item \code{countsData()}: Accesses the \code{counts} data matrix.
#' \item \code{weightsData()}: Accesses the (optional) \code{weights} data matrix.
#' \item \code{rowData()}: Accesses the \code{DataFrame} of row meta-data. This should
#' contain two columns: \code{gene_IDs} and \code{exon_IDs}.
#' \item \code{colData()}: Accesses the \code{DataFrame} of column meta-data. This
#' contains the experimental condition and (optionally) normalization factors for each
#' biological sample.
#' }
#' 
#' 
#' @section Subsetting:
#' 
#' Subsetting of \code{RegspliceData} objects is performed with square brackets, 
#' \code{x[i, j]}, where \code{x} is the name of the object. The subsetting operations
#' are designed to keep data and meta-data in sync.
#' 
#' For subsetting by rows, there are two possibilities:
#' \itemize{
#' \item Subsetting genes: To subset whole genes, provide a character vector of gene 
#' names to the argument \code{i}. The returned object will contain all rows 
#' corresponding to these genes. Row numbers should not be used when subsetting whole
#' genes, since this risks potential errors due to selecting incorrect rows.
#' \item Subsetting exons: To subset individual exons, provide the corresponding row
#' numbers to the argument \code{i}.
#' }
#' 
#' For subsetting by columns (biological samples), provide the corresponding column
#' numbers to the argument \code{j}.
#' 
#' 
#' @return Returns a \code{RegspliceData} object.
#' 
#' @seealso \code{\link{regsplice}} \code{\link{filterZeros}}
#' 
#' @importFrom S4Vectors DataFrame SimpleList
#' @importFrom SummarizedExperiment SummarizedExperiment Assays
#' @importFrom methods new
#' 
#' @export
#' 
#' @examples
#' # ---------
#' # Example 1
#' # ---------
#' 
#' counts <- matrix(sample(100:200, 14 * 6, replace = TRUE), nrow = 14, ncol = 6)
#' gene_IDs <- paste0("gene", 1:5)
#' n_exons <- c(3, 2, 3, 1, 5)
#' condition <- rep(c(0, 1), each = 3)
#' 
#' rs_data <- RegspliceData(counts, gene_IDs, n_exons, condition)
#' 
#' rs_data
#' countsData(rs_data)
#' rowData(rs_data)
#' colData(rs_data)
#' 
#' rs_data[1, ]
#' rs_data[1, 1:3]
#' 
#' rs_data["gene1", ]
#' rs_data["gene1", 1:3]
#' 
#' 
#' # --------------------
#' # Example 2 (Vignette)
#' # --------------------
#' 
#' file_counts <- system.file("extdata/vignette_counts.txt", package = "regsplice")
#' data <- read.table(file_counts, header = TRUE, sep = "\t", stringsAsFactors = FALSE)
#' head(data)
#' 
#' counts <- data[, 2:7]
#' tbl_exons <- table(sapply(strsplit(data$exon, ":"), function(s) s[[1]]))
#' gene_IDs <- names(tbl_exons)
#' n_exons <- unname(tbl_exons)
#' condition <- rep(c("untreated", "treated"), each = 3)
#' 
#' rs_data <- RegspliceData(counts, gene_IDs, n_exons, condition)
#' 
#' rs_data
#' head(countsData(rs_data))
#' rowData(rs_data)
#' colData(rs_data)
#' 
#' rs_data[1, ]
#' rs_data[1, 1:3]
#' 
#' rs_data["ENSG00000000003", ]
#' rs_data["ENSG00000000003", 1:3]
#' 
RegspliceData <- function(counts, gene_IDs = NULL, n_exons = NULL, condition = NULL) {
  
  # extract components if input is provided as a SummarizedExperiment object
  
  if (is(counts, "SummarizedExperiment")) {
    
    se <- counts
    counts <- assays(se)$counts
    gene_IDs <- unique(rowData(se)$gene_IDs)
    n_exons <- unname(table(rowData(se)$gene_IDs))
    condition <- colData(se)$condition
    
    if (length(assays(se)) > 1) {
      warning("SummarizedExperiment input object contains more than one 'assays' object. ", 
              "Only the first 'assays' object will be used; this is assumed to contain the read counts.")
    }
    if (!all(gene_IDs == names(table(rowData(se)$gene_IDs)))) {
      stop("gene IDs do not match (this may be due to alphabetical re-ordering)")
    }
    if (is.null(counts)) {
      stop("'counts' matrix could not be identified in SummarizedExperiment input object")
    }
    if (is.null(gene_IDs)) {
      stop("'gene_IDs' vector could not be found in rowData of SummarizedExperiment input object")
    }
    if (is.null(n_exons)) {
      stop("number of exons per gene could not be calculated from repeated entries in 'gene_IDs' vector")
    }
    if (is.null(condition)) {
      stop("'condition' vector could not be found in colData of SummarizedExperiment input object")
    }
    
    
  } else {
    if (is.null(gene_IDs) | is.null(n_exons) | is.null(condition)) {
      stop("please provide all required inputs: ", 
           "either a SummarizedExperiment object, ", 
           "or all of 'counts', 'gene_IDs', 'n_exons', and 'condition'")
    }
  }
  
  
  # construct RegspliceData object
  
  if (!(is.matrix(counts) | is.data.frame(counts) | is(counts, "DataFrame"))) {
    stop("'counts' must be a matrix, data.frame, or DataFrame (S4Vectors)")
  }
  if (!is.character(gene_IDs)) {
    stop("'gene_IDs' must be a character vector")
  }
  if (!is.numeric(n_exons)) {
    stop("'n_exons' must be a numeric vector")
  }
  if (!(is.character(condition) | is.numeric(condition) | is.factor(condition))) {
    stop("'condition' must be a character vector, numeric vector, or factor")
  }
  
  if (sum(n_exons) != nrow(counts)) {
    stop("total number of exons 'sum(n_exons)' does not match number of rows in 'counts'")
  }
  if (length(condition) != ncol(counts)) {
    stop("number of samples (length of 'condition' vector) does not match number of columns in 'counts'")
  }
  
  gene_IDs_rep <- unname(as.vector(unlist(mapply(rep, gene_IDs, n_exons))))
  
  # generate exon IDs as 3-digit numbers with leading zeros
  exon_IDs <- sprintf("%03d", unlist(lapply(n_exons, seq)))
  
  rowData <- S4Vectors::DataFrame(gene_IDs = gene_IDs_rep, exon_IDs = exon_IDs)
  
  if (!is.null(colnames(counts))) {
    sample_names <- colnames(counts)
    colData <- S4Vectors::DataFrame(sample_names = sample_names, condition = condition)
  } else {
    colData <- S4Vectors::DataFrame(condition = condition)
  }
  
  counts <- SummarizedExperiment::Assays(S4Vectors::SimpleList(counts = as.matrix(counts)))
  
  new("RegspliceData", 
      assays = counts, 
      elementMetadata = rowData, 
      colData = colData)
}




##########################
### Accessor functions ###
##########################


#' @rdname RegspliceData
#' @importFrom SummarizedExperiment assays "assays<-"
#' @importFrom methods callNextMethod
#' @export
#' 
setMethod("assays", "RegspliceData", function(x, withDimnames, ..., value) {
  # note: "value" is required for setter operation "assays<-"
  callNextMethod(x, withDimnames, ..., value)
})




#' @rdname RegspliceData
#' @export
#' 
setGeneric("countsData", function(x) {
  standardGeneric("countsData")
})


#' @rdname RegspliceData
#' @importFrom SummarizedExperiment assays
#' @export
#' 
setMethod("countsData", "RegspliceData", function(x) {
  SummarizedExperiment::assays(x)$counts
})




#' @rdname RegspliceData
#' @export
#' 
setGeneric("weightsData", function(x) {
  standardGeneric("weightsData")
})


#' @rdname RegspliceData
#' @importFrom SummarizedExperiment assays
#' @export
#' 
setMethod("weightsData", "RegspliceData", function(x) {
  SummarizedExperiment::assays(x)$weights
})




#' @rdname RegspliceData
#' @importFrom SummarizedExperiment rowData
#' @importFrom methods callNextMethod
#' @export
#' 
setMethod("rowData", "RegspliceData", function(x) {
  callNextMethod(x)
})




#' @rdname RegspliceData
#' @importFrom SummarizedExperiment colData "colData<-"
#' @importFrom methods callNextMethod
#' @export
#' 
setMethod("colData", "RegspliceData", function(x, ..., value) {
  # note: "value" is required for setter operation "colData<-"
  callNextMethod(x, ..., value)
})




###########################
### Subsetting function ###
###########################


#' @rdname RegspliceData
#' @importFrom methods callNextMethod
#' @export
#' 
setMethod("[", "RegspliceData", function(x, i, j) {
  
  if (missing(i)) {
    i <- seq_len(nrow(x))
  }
  
  if (is.character(i)) {
    # subsetting rows by gene names
    keep_i <- rowData(x)$gene_IDs %in% i
    callNextMethod(x = x, i = keep_i, j = j)

  } else {
    # subsetting rows by row numbers
    callNextMethod()
  }
})

library(regsplice)
context("glmnet error example")

test_that("glmnet error example passes", {
  
  # 'glmnet' version 2.0-2 introduced a bug in the cross validation step, which gives 
  # errors in the 'regsplice' lasso model fitting function '.fitRegSingle()' for some 
  # random seeds. Earlier versions of 'glmnet' (<= v. 1.9-8) did not have this problem. 
  # The 'glmnet' package authors have advised that the bug will be fixed in a future 
  # update. Until then, we have included checks in '.fitRegSingle()' to simply re-run 
  # the model fitting procedure if the error occurs. This unit test checks that the 
  # 'regsplice' functions pass for a specific example of the bug. For more details, see 
  # the GitHub repository at: https://github.com/lmweber/glmnet-error-example
  
  # LOAD DATA
  # Y: vector of response values
  # X: design matrix of indicator variables and interaction terms
  file <- system.file("extdata/data_glmnet_error_example.txt", package = "regsplice")
  data <- read.table(file, header = TRUE, sep = "\t", check.names = FALSE)
  Y <- data[, 1]
  X <- as.matrix(data[, -1])
  
  # PENALTY FACTOR
  # penalize interaction terms only
  penalty <- rep(c(0, 1), times = c(13, 8))
  
  # SHOW ERROR BY RUNNING CV.GLMNET DIRECTLY (not run)
  # cv.glmnet: cross validation for lambda
  # The following code demonstrates the error.
  # error message: 'Error in predmat[which, seq(nlami)] = preds : replacement has length zero'
  #set.seed(1)
  #glmnet::cv.glmnet(x = X, y = Y, family = "gaussian", penalty.factor = penalty)
  
  # RUN REGSPLICE MODEL FITTING FUNCTION
  # regsplice model fitting function checks for the error and should pass
  counts <- matrix(Y, ncol = 6)
  gene_IDs <- "gene1"
  n_exons <- 9
  condition <- rep(c(0, 1), each = 3)
  
  rs_data <- RegspliceData(counts, gene_IDs, n_exons, condition)
  
  # filtering low-count exons can also solve the problem
  #rs_data <- filterZeros(rs_data)
  #rs_data <- filterLowCounts(rs_data)
  
  rs_results <- initializeResults(rs_data)
  
  # error occurs for set.seed(1) if checks are not included in '.fitRegSingle()'
  set.seed(1)
  rs_results <- fitRegMultiple(rs_results, rs_data)
  
  
  # no output object is returned if the error occurs
  expect_is(rs_results, "RegspliceResults")
  expect_length(rs_results@fit_reg_dev, 1)
  expect_length(rs_results@fit_reg_df, 1)
  
})

library(regsplice)
context("regsplice wrapper function")

test_that("'regsplice()' wrapper function gives correct results", {
  
  set.seed(123)
  n_exons <- c(7, 18, 5, 5, 1, 3, 11)
  counts <- matrix(sample(100:200, sum(n_exons) * 6, replace = TRUE), ncol = 6)
  gene_IDs <- paste0("gene", 1:length(n_exons))
  condition <- rep(c(0, 1), each = 3)
  
  rs_data <- RegspliceData(counts, gene_IDs, n_exons, condition)
  
  rs_results <- regsplice(rs_data)
  
  
  n <- 6  # expect 6 remaining genes
  
  expect_length(rs_results@gene_IDs, n)
  
  expect_length(rs_results@p_vals, n)
  expect_length(rs_results@p_adj, n)
  expect_length(rs_results@LR_stats, n)
  expect_length(rs_results@df_tests, n)
  
  expect_true(all(!is.na(rs_results@p_vals)))
  expect_true(all(!is.na(rs_results@p_adj)))
  
  expect_true(all(rs_results@p_vals >= 0))
  expect_true(all(rs_results@p_vals <= 1))
  expect_true(all(rs_results@p_adj >= 0))
  expect_true(all(rs_results@p_adj <= 1))
  
  expect_true(all(rs_results@LR_stats >= 0, na.rm = TRUE))
  expect_true(all(rs_results@df_tests >= 1, na.rm = TRUE))
})

library(regsplice)
context("Input data preparation")

test_that("error if number of rows in count table does not match total number of exons", {
  
  set.seed(123)
  counts <- matrix(sample(100:200, 7 * 4, replace = TRUE), nrow = 7, ncol = 4)
  n_exons <- c(3, 1, 2)
  gene_IDs <- paste0("gene", 1:length(n_exons))
  condition <- rep(c(0, 1), each = 2)
  
  expect_error(RegspliceData(counts, gene_IDs, n_exons, condition), "total number of exons")
})




test_that("genes containing only a single exon are removed", {
  
  set.seed(123)
  n_exons <- c(11, 1, 3, 1, 7)
  counts <- matrix(sample(100:200, sum(n_exons) * 6, replace = TRUE), ncol = 6)
  gene_IDs <- paste0("gene", 1:length(n_exons))
  condition <- rep(c(0, 1), each = 3)
  
  rs_data <- RegspliceData(counts, gene_IDs, n_exons, condition)
  rs_data <- filterZeros(rs_data)
  rs_data <- filterLowCounts(rs_data)
  
  n_genes <- length(table(rowData(rs_data)$gene_IDs))
  n_exons_total <- nrow(rowData(rs_data))
  
  expect_equal(n_genes, 3)
  expect_equal(n_exons_total, 21)
})




test_that("genes with all zero counts are removed", {
  
  counts <- rbind(matrix(1, nrow = 5, ncol = 4), 
                  matrix(0, nrow = 3, ncol = 4), 
                  matrix(1, nrow = 2, ncol = 4))
  gene_IDs <- paste0("gene", 1:4)
  n_exons <- c(3, 2, 3, 2)
  condition <- rep(c(0, 1), each = 2)
  
  rs_data <- RegspliceData(counts, gene_IDs, n_exons, condition)
  rs_data <- filterZeros(rs_data)
  
  n_genes <- length(table(rowData(rs_data)$gene_IDs))
  n_exons_total <- nrow(rowData(rs_data))
  
  expect_equal(n_genes, 3)
  expect_equal(n_exons_total, 7)
})

library(regsplice)
context("when_null_selected options")

test_that('when_null_selected option "ones" works correctly', {
  
  n_exons <- c(7, 18, 5, 5, 1, 3, 11)
  set.seed(123)
  counts <- matrix(sample(100:200, sum(n_exons) * 6, replace = TRUE), ncol = 6)
  gene_IDs <- paste0("gene", 1:length(n_exons))
  condition <- rep(c(0, 1), each = 3)
  
  rs_data <- RegspliceData(counts, gene_IDs, n_exons, condition)
  
  rs_results <- regsplice(rs_data, when_null_selected = "ones")
  
  
  n <- 6  # expect 6 remaining genes
  
  expect_length(rs_results@gene_IDs, n)
  
  expect_length(rs_results@p_vals, n)
  expect_length(rs_results@p_adj, n)
  
  expect_true(sum(rs_results@p_vals == 1) > 0)
  expect_true(sum(rs_results@p_adj == 1) > 0)
})




test_that('when_null_selected option "full" works correctly', {
  
  n_exons <- c(7, 18, 5, 5, 1, 3, 11)
  set.seed(123)
  counts <- matrix(sample(100:200, sum(n_exons) * 6, replace = TRUE), ncol = 6)
  gene_IDs <- paste0("gene", 1:length(n_exons))
  condition <- rep(c(0, 1), each = 3)
  
  rs_data <- RegspliceData(counts, gene_IDs, n_exons, condition)
  
  rs_results <- regsplice(rs_data, when_null_selected = "full")
  
  
  n <- 6  # expect 6 remaining genes
  
  expect_length(rs_results@gene_IDs, n)
  
  expect_length(rs_results@p_vals, 6)
  expect_length(rs_results@p_adj, 6)
  
  expect_true(sum(is.na(rs_results@p_vals)) == 0)
  expect_true(sum(is.na(rs_results@p_adj)) == 0)
})




test_that('when_null_selected option "NA" works correctly', {
  
  n_exons <- c(7, 18, 5, 5, 1, 3, 11)
  set.seed(123)
  counts <- matrix(sample(100:200, sum(n_exons) * 6, replace = TRUE), ncol = 6)
  gene_IDs <- paste0("gene", 1:length(n_exons))
  condition <- rep(c(0, 1), each = 3)
  
  rs_data <- RegspliceData(counts, gene_IDs, n_exons, condition)
  
  rs_results <- regsplice(rs_data, when_null_selected = "NA")
  
  
  n <- 6  # expect 6 remaining genes
  
  expect_length(rs_results@gene_IDs, n)
  
  expect_length(rs_results@p_vals, 6)
  expect_length(rs_results@p_adj, 6)
  
  expect_true(sum(is.na(rs_results@p_vals)) > 0)
  expect_true(sum(is.na(rs_results@p_adj)) > 0)
})

library(regsplice)
context("Filter low-count exons")

test_that("low-count exon bins are filtered correctly", {
  
  counts <- rbind(c(1, 0, 0, 0, 0, 0), 
                  c(2, 3, 5, 1, 2, 1), 
                  c(0, 2, 1, 1, 2, 2), 
                  c(0, 4, 0, 0, 0, 0), 
                  c(0, 3, 4, 1, 1, 2), 
                  c(10, 5, 6, 8, 2, 8), 
                  c(13, 8, 9, 2, 4, 2))
  gene_IDs <- paste0("gene", 1:3)
  n_exons <- c(3, 2, 2)
  condition <- rep(c(0, 1), each = 3)
  
  rs_data <- RegspliceData(counts, gene_IDs, n_exons, condition)
  
  rs_data <- filterZeros(rs_data)
  rs_data <- filterLowCounts(rs_data)
  
  n_genes <- length(names(table(rowData(rs_data)$gene_IDs)))
  n_exons <- nrow(rowData(rs_data))
  
  expect_equivalent(n_genes, 1)
  expect_equivalent(n_exons, 2)
})

library(regsplice)
context("voom transformation and weights")

test_that("voom transformation and weights are calculated correctly", {
  
  counts <- cbind(sample1 = c(1, 4, 0, 5), 
                  sample2 = c(0, 1, 0, 5), 
                  sample3 = c(0, 9, 6, 30), 
                  sample4 = c(3, 12, 4, 23), 
                  sample5 = c(1, 4, 0, 10), 
                  sample6 = c(1, 3, 1, 22))
  gene_IDs <- "gene1"
  n_exons <- 4
  condition <- rep(c(0, 1), each = 3)
  
  rs_data <- RegspliceData(counts, gene_IDs, n_exons, condition)
  rs_data <- filterZeros(rs_data)
  rs_data <- filterLowCounts(rs_data)
  rs_data <- runNormalization(rs_data)
  rs_data <- runVoom(rs_data)
  
  lib_sizes <- colData(rs_data)$lib_sizes
  
  
  expect_length(lib_sizes, 6)
  
  expect_is(weightsData(rs_data), "matrix")
  expect_is(weightsData(rs_data)[2, 3], "numeric")
})

library(regsplice)
context("subsetting RegspliceData objects")

test_that("row-subsetting RegspliceData objects", {
  
  set.seed(123)
  n_exons <- c(7, 18, 5, 5, 1, 3, 11)
  counts <- matrix(sample(100:200, sum(n_exons) * 6, replace = TRUE), ncol = 6)
  gene_IDs <- paste0("gene", 1:length(n_exons))
  condition <- rep(c(0, 1), each = 3)
  
  rs_data <- RegspliceData(counts, gene_IDs, n_exons, condition)
  
  # row-subsetting using row index
  expect_equal(nrow(rs_data[1, ]), 1)
  expect_equal(nrow(rs_data[1:3, ]), 3)
  
  # row-subsetting using gene IDs
  expect_equal(nrow(rs_data["gene1", ]), 7)
  expect_equal(nrow(rs_data["gene2", ]), 18)
})




test_that("column-subsetting RegspliceData objects", {
  
  set.seed(123)
  n_exons <- c(7, 18, 5, 5, 1, 3, 11)
  counts <- matrix(sample(100:200, sum(n_exons) * 6, replace = TRUE), ncol = 6)
  gene_IDs <- paste0("gene", 1:length(n_exons))
  condition <- rep(c(0, 1), each = 3)
  
  rs_data <- RegspliceData(counts, gene_IDs, n_exons, condition)
  
  # column-subsetting
  expect_equal(ncol(rs_data[, 1:2]), 2)
  expect_equal(ncol(rs_data[, 4:6]), 3)
  expect_equal(ncol(rs_data[, 6]), 1)
})




test_that("row-and-column-subsetting RegspliceData objects", {
  
  set.seed(123)
  n_exons <- c(7, 18, 5, 5, 1, 3, 11)
  counts <- matrix(sample(100:200, sum(n_exons) * 6, replace = TRUE), ncol = 6)
  gene_IDs <- paste0("gene", 1:length(n_exons))
  condition <- rep(c(0, 1), each = 3)
  
  rs_data <- RegspliceData(counts, gene_IDs, n_exons, condition)
  
  # subsetting both rows and columns
  
  expect_equal(nrow(rs_data[1, 1]), 1)
  expect_equal(ncol(rs_data[1, 1]), 1)
  
  expect_equal(nrow(rs_data[2, 3]), 1)
  expect_equal(ncol(rs_data[2, 3]), 1)
  
  expect_equal(nrow(rs_data[5:10, 4:6]), 6)
  expect_equal(ncol(rs_data[5:10, 4:6]), 3)
  
  expect_equal(nrow(rs_data["gene1", 1:3]), 7)
  expect_equal(ncol(rs_data["gene1", 1:3]), 3)
  
  expect_equal(nrow(rs_data[c("gene3", "gene6"), 4]), 8)
  expect_equal(ncol(rs_data[c("gene3", "gene6"), 4]), 1)
  
  expect_equal(nrow(rs_data[c("gene1", "gene2"), 1:3]), 25)
  expect_equal(ncol(rs_data[c("gene1", "gene2"), 1:3]), 3)
})

library(regsplice)
context("Input data formats")

test_that("counts matrix can be input as matrix or data frame", {
  
  counts <- matrix(1, nrow = 5, ncol = 6)
  counts_df <- as.data.frame(counts)
  gene_IDs <- c("gene1", "gene2")
  n_exons <- c(3, 2)
  condition <- rep(c(0, 1), each = 3)
  
  rs_data_matrix <- RegspliceData(counts,    gene_IDs, n_exons, condition)
  rs_data_df     <- RegspliceData(counts_df, gene_IDs, n_exons, condition)
  
  se <- SummarizedExperiment::SummarizedExperiment(
    assays = S4Vectors::SimpleList(counts = counts), 
    rowData = S4Vectors::DataFrame(gene_IDs = rep(gene_IDs, n_exons)), 
    colData = S4Vectors::DataFrame(condition))
  
  rs_se <- RegspliceData(se)
  
  expect_equivalent(rs_data_matrix, rs_data_df)
  expect_equivalent(rs_data_matrix, rs_se)
  expect_equivalent(rs_data_df, rs_se)
})

library(regsplice)
context("Vignette example")

test_that("results from vignette example are as expected", {
  
  file_counts <- system.file("extdata/vignette_counts.txt", package = "regsplice")
  data <- read.table(file_counts, header = TRUE, sep = "\t", stringsAsFactors = FALSE)
  
  counts <- data[, 2:7]
  tbl_exons <- table(sapply(strsplit(data$exon, ":"), function(s) s[[1]]))
  gene_IDs <- names(tbl_exons)
  n_exons <- unname(tbl_exons)
  condition <- rep(c("untreated", "treated"), each = 3)
  
  # run wrapper function
  # note: suppress warnings for grouped = FALSE in cv.glmnet due to small number of observations
  rs_data <- RegspliceData(counts, gene_IDs, n_exons, condition)
  suppressWarnings(
    rs_results <- regsplice(rs_data)
  )
  
  n_genes <- length(rs_results@gene_IDs)
  
  n <- 81  # expect 81 genes remaining (out of 100 initially)
  
  
  expect_equal(n_genes, n)
  
  expect_length(rs_results@p_vals, n)
  expect_length(rs_results@p_adj, n)
  expect_length(rs_results@LR_stats, n)
  expect_length(rs_results@df_tests, n)
  
  expect_true(all(!is.na(rs_results@p_vals)))
  expect_true(all(!is.na(rs_results@p_adj)))
  
  expect_true(all(rs_results@p_vals >= 0))
  expect_true(all(rs_results@p_vals <= 1))
  expect_true(all(rs_results@p_adj >= 0))
  expect_true(all(rs_results@p_adj <= 1))
  
  expect_true(all(rs_results@LR_stats >= 0, na.rm = TRUE))
  expect_true(all(rs_results@df_tests >= 1, na.rm = TRUE))
})

library(regsplice)
context("Normalization factors")

test_that("normalization factors work correctly", {
  
  counts <- rbind(c(101, 198, 53, 102, 100, 101), 
                  c(90, 220, 60, 89, 90, 103), 
                  c(95, 205, 66, 105, 85, 99), 
                  c(300, 622, 145, 311, 290, 320), 
                  c(290, 602, 140, 290, 303, 298), 
                  c(290, 410, 90, 190, 211, 209))
  gene_IDs <- paste0("gene", 1:2)
  n_exons <- c(3, 3)
  condition <- rep(c(0, 1), each = 3)
  
  rs_data <- RegspliceData(counts, gene_IDs, n_exons, condition)
  rs_data <- filterZeros(rs_data)
  rs_data <- filterLowCounts(rs_data)
  rs_data <- runNormalization(rs_data)
  
  norm_factors <- colData(rs_data)$norm_factors
  
  expect_length(norm_factors, 6)
  expect_is(norm_factors, "numeric")
})

#' @include class_RegspliceData.R class_RegspliceResults.R
NULL




#' Wrapper function to run regsplice.
#' 
#' Wrapper function to run a \code{regsplice} analysis with a single command.
#' 
#' This wrapper function runs the \code{regsplice} analysis pipeline with a single 
#' command.
#' 
#' The required input format is a \code{RegspliceData} object, which is created with the 
#' \code{\linkS4class{RegspliceData}} constructor function.
#' 
#' The wrapper function calls each of the individual functions in the analysis pipeline 
#' in sequence. You can also run the individual functions directly, which provides 
#' additional flexibility and insight into the statistical methodology. See the vignette 
#' for a description of the individual functions and an example workflow.
#' 
#' After running the analysis pipeline, a summary table of the results can be displayed 
#' with \code{\link{summaryTable}}.
#' 
#' Note that when using exon microarray data, the filtering, normalization, and 
#' \code{voom} steps should be disabled with the respective arguments.
#' 
#' See \code{\linkS4class{RegspliceData}} for details on constructing the input data 
#' object; \code{\link{filterZeros}} and \code{\link{filterLowCounts}} for details about
#' filtering; \code{\link{runNormalization}} and \code{\link{runVoom}} for details about
#' calculation of normalization factors and \code{voom} transformation and weights;
#' \code{\link{createDesignMatrix}} for details about the model design matrices;
#' \code{\link{fitRegMultiple}}, \code{\link{fitNullMultiple}}, or 
#' \code{\link{fitFullMultiple}} for details about the model fitting functions; and 
#' \code{\link{LRTests}} for details about the likelihood ratio tests.
#' 
#' 
#' @param rs_data \code{RegspliceData} object containing input data. See 
#'   \code{\linkS4class{RegspliceData}} for details.
#' @param filter_zeros Whether to filter zero-count exon bins, using 
#'   \code{\link{filterZeros}}. Default is TRUE. Set to FALSE for exon microarray data.
#' @param filter_low_counts Whether to filter low-count exon bins, using 
#'   \code{\link{filterLowCounts}}. Default is TRUE. Set to FALSE for exon microarray 
#'   data.
#' @param filter_min_per_exon Filtering parameter for low-count exon bins: minimum number
#'   of reads per exon bin, summed across all biological samples. Default is 6. See 
#'   \code{\link{filterLowCounts}} for details.
#' @param filter_min_per_sample Filtering parameter for low-count exon bins: minimum 
#'   number of reads per biological sample; i.e. for each exon bin, at least one sample 
#'   must have this number of reads. Default is 3. See \code{\link{filterLowCounts}} for
#'   details.
#' @param normalize Whether to calculate normalization factors, using 
#'   \code{\link{runNormalization}}. Default is TRUE. If FALSE, non-normalized library 
#'   sizes will be used. Set to FALSE for exon microarray data.
#' @param norm_method Normalization method to use. Options are \code{"TMM"}, 
#'   \code{"RLE"}, \code{"upperquartile"}, and \code{"none"}. Default is \code{"TMM"}. 
#'   See \code{\link{runNormalization}} for details.
#' @param voom Whether to calculate \code{limma-voom} transformation and weights, using 
#'   \code{\link{runVoom}}. Default is TRUE. If FALSE, model fitting functions will use 
#'   the raw input data (not recommended for count data) with exon bins weighted equally.
#'   Set to FALSE for exon microarray data.
#' @param alpha Elastic net parameter \code{alpha} for \code{glmnet} model fitting 
#'   functions. Must be between 0 (ridge regression) and 1 (lasso). Default is 1 (lasso).
#'   See \code{glmnet} documentation for more details.
#' @param lambda_choice Parameter to select which optimal \code{lambda} value to choose 
#'   from the \code{cv.glmnet} cross validation fit. Choices are "lambda.min" (model with
#'   minimum cross-validated error) and "lambda.1se" (most regularized model with 
#'   cross-validated error within one standard error of minimum). Default is 
#'   "lambda.min". See \code{glmnet} documentation for more details.
#' @param when_null_selected Which option to use for genes where the lasso model selects 
#'   zero interaction terms, i.e. identical to the null model. Options are \code{"ones"},
#'   \code{"full"}, and \code{"NA"}. Default is \code{"ones"}. See \code{\link{LRTests}} 
#'   for details.
#' @param seed Random seed (integer). Default is NULL. Provide an integer value to set 
#'   the random seed for reproducible results.
#' @param ... Other arguments to pass to \code{cv.glmnet}, \code{glmnet}, or \code{glm}.
#' 
#' 
#' @return Returns a \code{\linkS4class{RegspliceResults}} object containing fitted model
#'   results and likelihood ratio (LR) test results. The LR test results consist of the 
#'   following entries for each gene:
#' \itemize{
#' \item p_vals: raw p-values
#' \item p_adj: multiple testing adjusted p-values (Benjamini-Hochberg false discovery 
#' rates, FDR)
#' \item LR_stats: likelihood ratio test statistics
#' \item df_tests: degrees of freedom of likelihood ratio tests
#' }
#' 
#' 
#' @seealso \code{\linkS4class{RegspliceData}} \code{\linkS4class{RegspliceResults}} 
#'   \code{\link{initializeResults}} \code{\link{filterZeros}} 
#'   \code{\link{filterLowCounts}} \code{\link{runNormalization}} \code{\link{runVoom}}
#'   \code{\link{createDesignMatrix}} \code{\link{fitRegMultiple}}
#'   \code{\link{fitNullMultiple}} \code{\link{fitFullMultiple}} \code{\link{LRTests}}
#'   \code{\link{summaryTable}}
#' 
#' @export
#'
#' @examples
#' file_counts <- system.file("extdata/vignette_counts.txt", package = "regsplice")
#' data <- read.table(file_counts, header = TRUE, sep = "\t", stringsAsFactors = FALSE)
#' head(data)
#' 
#' counts <- data[, 2:7]
#' tbl_exons <- table(sapply(strsplit(data$exon, ":"), function(s) s[[1]]))
#' gene_IDs <- names(tbl_exons)
#' n_exons <- unname(tbl_exons)
#' condition <- rep(c("untreated", "treated"), each = 3)
#' 
#' rs_data <- RegspliceData(counts, gene_IDs, n_exons, condition)
#' 
#' rs_results <- regsplice(rs_data)
#' 
#' summaryTable(rs_results)
#' 
regsplice <- function(rs_data, 
                      filter_zeros = TRUE, filter_low_counts = TRUE, 
                      filter_min_per_exon = 6, filter_min_per_sample = 3, 
                      normalize = TRUE, norm_method = "TMM", voom = TRUE, 
                      alpha = 1, lambda_choice = c("lambda.min", "lambda.1se"), 
                      when_null_selected = c("ones", "full", "NA"), 
                      seed = NULL, ...) {
  
  lambda_choice <- match.arg(lambda_choice)
  when_null_selected <- match.arg(when_null_selected)
  
  if (filter_zeros) rs_data <- filterZeros(rs_data)
  
  if (filter_low_counts) {
    rs_data <- filterLowCounts(rs_data, 
                               filter_min_per_exon = filter_min_per_exon, 
                               filter_min_per_sample = filter_min_per_sample)
  }
  
  if (normalize) rs_data <- runNormalization(rs_data, 
                                             norm_method = norm_method)
  
  if (voom) rs_data <- runVoom(rs_data)
  
  rs_results <- initializeResults(rs_data)
  
  rs_results <- fitRegMultiple(rs_results, 
                               rs_data, 
                               alpha = alpha, 
                               lambda_choice = lambda_choice, 
                               seed = seed, 
                               ...)
  
  rs_results <- fitNullMultiple(rs_results, 
                                rs_data, 
                                seed = seed, 
                                ...)
  
  if (when_null_selected == "full") {
    rs_results <- fitFullMultiple(rs_results, 
                                  rs_data, 
                                  seed = seed, 
                                  ...)
  }
  
  rs_results <- LRTests(rs_results, 
                        when_null_selected = when_null_selected)
  
  rs_results
}

# Model fitting functions for a single gene. These are internal functions, which should 
# not be called directly by users.


#' @importFrom glmnet cv.glmnet
#' @importFrom stats deviance
#' @importFrom methods is
#' 
.fitRegSingle <- function(rs_data, alpha = 1, 
                          lambda_choice = c("lambda.min", "lambda.1se"), ...) {
  
  if (!("RegspliceData" %in% is(rs_data))) stop("'rs_data' must be a 'RegspliceData' object")
  
  Y <- countsData(rs_data)
  weights <- weightsData(rs_data)
  condition <- colData(rs_data)$condition
  
  n_exons <- nrow(Y)
  X <- createDesignMatrix(condition = condition, n_exons = n_exons)
  if (is.null(weights)) weights <- matrix(1, nrow = n_exons, ncol = length(condition))
  
  lambda_choice <- match.arg(lambda_choice)
  
  # convert Y and weights to vectors (note matrix is read by column)
  Y <- as.vector(Y)
  weights <- as.vector(weights)
  
  # identify interaction columns by ":" in column names
  int_cols <- grepl(":", colnames(X))
  
  # which columns to penalize during lasso fitting
  pen <- as.numeric(int_cols)
  
  # Note: 'while' loop and 'tryCatch' statement are included to repeat the model fitting
  # procedure if 'cv.glmnet' returns an error, due to the bug introduced in 'glmnet'
  # version 2.0-2. See unit test 'test_glmnet_error_example.R' or GitHub repository at 
  # https://github.com/lmweber/glmnet-error-example for details. Since the error depends 
  # on the random seed, we simply re-run the model fitting procedure (with changing 
  # random seed) until it passes (maximum of 20 tries).
  fit <- NULL
  i <- 0
  while (is.null(fit) & i < 20) {
    tryCatch(fit <- glmnet::cv.glmnet(x = X, y = Y, weights = weights, 
                                      alpha = alpha, penalty.factor = pen, 
                                      intercept = TRUE, standardize = FALSE, ...), 
             error = function(e) {
               # print any unrelated error messages, which do not contain the term 'predmat'
               if (!grepl("predmat", e)) print(as.character(e))
             })
    i <- i + 1
  }
  
  if (is.null(fit)) {
    dev <- NA
    df <- NA
  } else {
    ix_opt <- which(fit$lambda == fit[[lambda_choice]])
    dev <- stats::deviance(fit$glmnet.fit)[ix_opt]
    df <- fit$glmnet.fit$df[ix_opt]
  }
  
  list(dev = dev, df = df)
}



#' @importFrom stats glm
#' @importFrom methods is
#' 
.fitNullSingle <- function(rs_data, ...) {
  
  if (!("RegspliceData" %in% is(rs_data))) stop("'rs_data' must be a 'RegspliceData' object")
  
  Y <- countsData(rs_data)
  weights <- weightsData(rs_data)
  condition <- colData(rs_data)$condition
  
  n_exons <- nrow(Y)
  X <- createDesignMatrix(condition = condition, n_exons = n_exons)
  if (is.null(weights)) weights <- matrix(1, nrow = n_exons, ncol = length(condition))
  
  # identify interaction columns by ":" in column names
  int_cols <- grepl(":", colnames(X))
  
  # convert Y and weights to vectors (note matrix is read by column)
  Y <- as.vector(Y)
  weights <- as.vector(weights)
  
  fit <- stats::glm(Y ~ X[, !int_cols], weights = weights, ...)
  
  dev <- fit$deviance
  df <- fit$df.null - fit$df.residual
  
  list(dev = dev, df = df)
}



#' @importFrom stats glm
#' @importFrom methods is
#' 
.fitFullSingle <- function(rs_data, ...) {
  
  if (!("RegspliceData" %in% is(rs_data))) stop("'rs_data' must be a 'RegspliceData' object")
  
  Y <- countsData(rs_data)
  weights <- weightsData(rs_data)
  condition <- colData(rs_data)$condition
  
  n_exons <- nrow(Y)
  X <- createDesignMatrix(condition = condition, n_exons = n_exons)
  if (is.null(weights)) weights <- matrix(1, nrow = n_exons, ncol = length(condition))
  
  # convert Y and weights to vectors (note matrix is read by column)
  Y <- as.vector(Y)
  weights <- as.vector(weights)
  
  fit <- stats::glm(Y ~ X, weights = weights, ...)
  
  dev <- fit$deviance
  df <- fit$df.null - fit$df.residual
  
  list(dev = dev, df = df)
}

#' @include class_RegspliceData.R class_RegspliceResults.R
NULL




#' Calculate likelihood ratio tests.
#' 
#' Calculate likelihood ratio tests between fitted models and null models.
#' 
#' The regularized (lasso) fitted models contain an optimal subset of exon:condition 
#' interaction terms for each gene, and the "full" fitted models contain all 
#' exon:condition interaction terms. The null models contain zero interaction terms, so 
#' they are nested within the fitted models.
#' 
#' The likelihood ratio (LR) tests compare the fitted models against the nested null 
#' models.
#' 
#' If the regularized (lasso) model contains at least one exon:condition interaction 
#' term, the LR test compares the lasso model against the null model. However, if the 
#' lasso model contains zero interaction terms, then the lasso and null models are 
#' identical, so the LR test cannot be calculated. The \code{when_null_selected} argument
#' lets the user choose what to do in these cases: either set p-values equal to 1 
#' (\code{when_null_selected = "ones"}); or calculate a LR test using the "full" model 
#' containing all exon:condition interaction terms (\code{when_null_selected = "full"}), 
#' which reduces power due to the larger number of terms, but allows the evidence for 
#' differential exon usage among these genes to be distinguished. You can also return 
#' \code{NA}s for these genes (\code{when_null_selected = "NA"}).
#' 
#' The default option is \code{when_null_selected = "ones"}. This simply calls all these 
#' genes non-significant, which in most cases is sufficient since we are more interested 
#' in genes with strong evidence for differential exon usage. However, if it is important
#' to rank the low-evidence genes in your data set, use the \code{when_null_selected = 
#' "full"} option.
#' 
#' If \code{when_null_selected = "ones"} or \code{when_null_selected = "NA"}, the "full" 
#' fitted models are not required.
#' 
#' Previous step: Fit models with \code{\link{fitRegMultiple}}, 
#' \code{\link{fitNullMultiple}}, and \code{\link{fitFullMultiple}}.
#' Next step: Generate summary table of results with \code{\link{summaryTable}}.
#' 
#' 
#' @param rs_results \code{\linkS4class{RegspliceResults}} object containing results 
#'   generated by \code{\link{fitRegMultiple}}, \code{\link{fitNullMultiple}}, and 
#'   \code{\link{fitFullMultiple}}. If \code{when_null_selected = "ones" or "NA"}, the 
#'   "full" models are not required. See \code{\linkS4class{RegspliceResults}} for 
#'   details.
#' @param when_null_selected Which option to use for genes where the lasso model selects 
#'   zero interaction terms, i.e. identical to the null model. Options are \code{"ones"},
#'   \code{"full"}, and \code{"NA"}. Default is \code{"ones"}. See below for details.
#' 
#' 
#' @return Returns a \code{\linkS4class{RegspliceResults}} object containing results of 
#'   the LR tests. The results consist of the following entries for each gene:
#' \itemize{
#' \item p_vals: raw p-values
#' \item p_adj: multiple testing adjusted p-values (Benjamini-Hochberg false discovery 
#' rates, FDR)
#' \item LR_stats: likelihood ratio test statistics
#' \item df_tests: degrees of freedom of likelihood ratio tests
#' }
#' 
#' @seealso \code{\link{RegspliceResults}} \code{\link{initializeResults}} 
#'   \code{\link{fitRegMultiple}} \code{\link{fitNullMultiple}} 
#'   \code{\link{fitFullMultiple}} \code{\link{summaryTable}}
#' 
#' @importFrom stats pchisq p.adjust
#' 
#' @export
#' 
#' @examples
#' file_counts <- system.file("extdata/vignette_counts.txt", package = "regsplice")
#' data <- read.table(file_counts, header = TRUE, sep = "\t", stringsAsFactors = FALSE)
#' head(data)
#' 
#' counts <- data[, 2:7]
#' tbl_exons <- table(sapply(strsplit(data$exon, ":"), function(s) s[[1]]))
#' gene_IDs <- names(tbl_exons)
#' n_exons <- unname(tbl_exons)
#' condition <- rep(c("untreated", "treated"), each = 3)
#' 
#' rs_data <- RegspliceData(counts, gene_IDs, n_exons, condition)
#' 
#' rs_data <- filterZeros(rs_data)
#' rs_data <- filterLowCounts(rs_data)
#' rs_data <- runNormalization(rs_data)
#' rs_data <- runVoom(rs_data)
#' 
#' rs_results <- initializeResults(rs_data)
#' 
#' rs_results <- fitRegMultiple(rs_results, rs_data)
#' rs_results <- fitNullMultiple(rs_results, rs_data)
#' rs_results <- fitFullMultiple(rs_results, rs_data)
#' 
#' rs_results <- LRTests(rs_results)
#' 
LRTests <- function(rs_results, when_null_selected = c("ones", "full", "NA")) {
  
  when_null_selected <- match.arg(when_null_selected)
  
  if (is.null(rs_results@fit_full_dev) & when_null_selected == "full") {
    stop("fitted 'full' models must be provided if when_null_selected = 'full'")
  }
  
  LR_stats <- abs(rs_results@fit_reg_dev - rs_results@fit_null_dev)
  df_tests <- abs(rs_results@fit_reg_df - rs_results@fit_null_df)
  
  # genes where lasso selected zero interaction terms (equivalent to null model); 
  # or NAs (where glmnet did not complete)
  ix_remove <- df_tests == 0 | is.na(df_tests)
  
  p_vals_keep <- stats::pchisq(LR_stats[!ix_remove], df_tests[!ix_remove], lower.tail = FALSE)
  
  p_vals <- p_adj <- rep(NA, length(rs_results@fit_reg_dev))
  
  if (when_null_selected == "ones") {
    p_vals[!ix_remove] <- p_vals_keep
    p_vals[ix_remove] <- 1
    
    # multiple testing adjustment for number of calculated p-values (i.e. non-ones)
    p_adj[!ix_remove] <- stats::p.adjust(p_vals_keep, method = "fdr")
    p_adj[ix_remove] <- 1
    
    LR_stats[ix_remove] <- NA
    df_tests[ix_remove] <- NA
    
  } else if (when_null_selected == "full") {
    LR_stats_full <- abs(rs_results@fit_full_dev - rs_results@fit_null_dev)
    df_tests_full <- abs(rs_results@fit_full_df - rs_results@fit_null_df)
    
    p_vals_full <- stats::pchisq(LR_stats_full, df_tests_full, lower.tail = FALSE)
    
    p_vals[!ix_remove] <- p_vals_keep
    p_vals[ix_remove] <- p_vals_full[ix_remove]
    
    # multiple testing adjustment for number of calculated p-values (i.e. all genes)
    p_adj <- stats::p.adjust(p_vals, method = "fdr")
    
    LR_stats[ix_remove] <- LR_stats_full[ix_remove]
    df_tests[ix_remove] <- df_tests_full[ix_remove]
    
  } else if (when_null_selected == "NA") {
    p_vals[!ix_remove] <- p_vals_keep
    
    # multiple testing adjustment for number of calculated p-values (i.e. non-NAs)
    p_adj[!ix_remove] <- stats::p.adjust(p_vals_keep, method = "fdr")
    
    LR_stats[ix_remove] <- NA
    df_tests[ix_remove] <- NA
  }
  
  rs_results@p_vals <- p_vals
  rs_results@p_adj <- p_adj
  rs_results@LR_stats <- LR_stats
  rs_results@df_tests <- df_tests
  
  rs_results
}

#' @include class_RegspliceData.R class_RegspliceResults.R
NULL




#' Filter zero-count exons.
#' 
#' Filter exons with zero RNA-seq read counts in all biological samples.
#' 
#' Removes exon bins with zero RNA-seq read counts in all biological samples. Any 
#' remaining single-exon genes (after filtering) are also removed (since differential
#' splicing requires multiple exon bins).
#' 
#' Input data is assumed to be in the form of a \code{RegspliceData} object. See 
#' \code{\link{RegspliceData}} for details.
#' 
#' After filtering zero-count exon bins, any remaining genes containing only a single
#' exon bin are also removed (since differential splicing requires multiple exon bins).
#' 
#' Filtering should be skipped when using exon microarray data. (When using the 
#' \code{regsplice} wrapper function, filtering can be disabled with the argument 
#' \code{filter = FALSE}).
#' 
#' Previous step: Create \code{RegspliceData} object with \code{\link{RegspliceData}}
#' constructor function.
#' Next step: Filter low-count exon bins with \code{\link{filterLowCounts}}.
#' 
#' 
#' @param rs_data \code{\linkS4class{RegspliceData}} object.
#' 
#' 
#' @return Returns a \code{\linkS4class{RegspliceData}} object.
#' 
#' @seealso \code{\linkS4class{RegspliceData}} \code{\link{filterLowCounts}}
#' 
#' @importFrom methods is
#' 
#' @export
#' 
#' @examples
#' file_counts <- system.file("extdata/vignette_counts.txt", package = "regsplice")
#' data <- read.table(file_counts, header = TRUE, sep = "\t", stringsAsFactors = FALSE)
#' head(data)
#' 
#' counts <- data[, 2:7]
#' tbl_exons <- table(sapply(strsplit(data$exon, ":"), function(s) s[[1]]))
#' gene_IDs <- names(tbl_exons)
#' n_exons <- unname(tbl_exons)
#' condition <- rep(c("untreated", "treated"), each = 3)
#' 
#' rs_data <- RegspliceData(counts, gene_IDs, n_exons, condition)
#' 
#' rs_data <- filterZeros(rs_data)
#' 
filterZeros <- function(rs_data) {
  
  if (!("RegspliceData" %in% is(rs_data))) stop("'rs_data' must be a 'RegspliceData' object")
  
  # remove exon bins (rows) with zero counts in all samples (columns)
  counts <- countsData(rs_data)
  ix_zeros <- apply(counts, MARGIN = 1, function(d) all(d == 0))
  
  message(paste("removed", sum(ix_zeros), "exon(s) with zero counts"))
  
  rs_data <- suppressMessages(rs_data[!ix_zeros, ])
  
  # remove any remaining single-exon genes after filtering
  .removeSingleExonGenes(rs_data)
}

#' @include class_RegspliceData.R class_RegspliceResults.R
NULL




#' Calculate 'voom' transformation and weights.
#' 
#' Use \code{limma-voom} to transform counts and calculate exon-level weights.
#' 
#' Raw counts do not fulfill the statistical assumptions required for linear modeling. 
#' The \code{limma-voom} methodology transforms counts to log2-counts per million 
#' (logCPM), and calculates exon-level weights based on the observed mean-variance 
#' relationship. Linear modeling methods can then be applied.
#' 
#' For more details, see the documentation for \code{\link[limma]{voom}} in the 
#' \code{limma} package.
#' 
#' Note that \code{voom} assumes that exon bins (rows) with zero or low counts have 
#' already been removed, so this step should be done after filtering with 
#' \code{\link{filterZeros}} and \code{\link{filterLowCounts}}.
#' 
#' Normalization factors can be provided in a column named \code{norm_factors} in the 
#' column meta-data (\code{colData} slot) of the \code{\linkS4class{RegspliceData}} 
#' object. These will be used by \code{voom} to calculate normalized library sizes. If 
#' normalization factors are not provided, \code{voom} will use non-normalized library 
#' sizes (columnwise total counts) instead.
#' 
#' The experimental conditions or group labels for each biological sample are assumed to 
#' be in a column named \code{condition} in the column meta-data (\code{colData} slot) of
#' the \code{\linkS4class{RegspliceData}} object. This column is created when the object 
#' is initialized with the \code{RegspliceData()} constructor function.
#' 
#' The transformed counts are stored in the updated \code{counts} matrix, which can be 
#' accessed with the \code{\link{countsData}} accessor function. The weights are stored 
#' in a new data matrix labeled \code{weights}, which can be accessed with the 
#' \code{\link{weightsData}} accessor function. In addition, the normalized library sizes
#' (if available) are stored in a new column named \code{lib_sizes} in the column 
#' meta-data (\code{colData} slot).
#' 
#' If you are using exon microarray data, this step should be skipped, since exon 
#' microarray intensities are already on a continuous scale.
#' 
#' Previous step: Calculate normalization factors with \code{\link{runNormalization}}.
#' Next step: Initialize \code{\linkS4class{RegspliceResults}} object with the
#' constructor function \code{RegspliceResults()}.
#' 
#' 
#' @param rs_data \code{\linkS4class{RegspliceData}} object, which has been filtered with
#'   \code{\link{filterZeros}} and \code{\link{filterLowCounts}}, and (optionally) 
#'   normalization factors added with \code{\link{runNormalization}}.
#' 
#' 
#' @return Returns a \code{\linkS4class{RegspliceData}} object. Transformed counts are 
#'   stored in the \code{counts} matrix, and weights are stored in a new \code{weights} 
#'   data matrix. The data matrices can be accessed with the accessor functions 
#'   \code{\link{countsData}} and \code{\link{weightsData}}.
#' 
#' @seealso \code{\link{runNormalization}} \code{\link{fitRegMultiple}} 
#'   \code{\link{fitNullMultiple}} \code{\link{fitFullMultiple}}
#' 
#' @importFrom limma voom
#' @importFrom stats model.matrix
#' 
#' @export
#' 
#' @examples
#' file_counts <- system.file("extdata/vignette_counts.txt", package = "regsplice")
#' data <- read.table(file_counts, header = TRUE, sep = "\t", stringsAsFactors = FALSE)
#' head(data)
#' 
#' counts <- data[, 2:7]
#' tbl_exons <- table(sapply(strsplit(data$exon, ":"), function(s) s[[1]]))
#' gene_IDs <- names(tbl_exons)
#' n_exons <- unname(tbl_exons)
#' condition <- rep(c("untreated", "treated"), each = 3)
#' 
#' rs_data <- RegspliceData(counts, gene_IDs, n_exons, condition)
#' 
#' rs_data <- filterZeros(rs_data)
#' rs_data <- filterLowCounts(rs_data)
#' rs_data <- runNormalization(rs_data)
#' rs_data <- runVoom(rs_data)
#' 
runVoom <- function(rs_data) {
  
  norm_factors <- colData(rs_data)$norm_factors
  condition <- colData(rs_data)$condition
  counts <- countsData(rs_data)
  
  # design matrix
  design <- stats::model.matrix(~ condition)
  
  # normalized library sizes
  if (!is.null(norm_factors)) {
    lib_sizes <- norm_factors * colSums(counts)
  } else {
    lib_sizes <- NULL
  }
  
  colData(rs_data)$lib_sizes <- lib_sizes
  
  # run voom
  out_voom <- limma::voom(counts = counts, design = design, lib.size = lib_sizes)
  
  assays(rs_data)$counts <- out_voom$E
  assays(rs_data)$weights <- out_voom$weights
  
  rs_data
}

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(collapse = TRUE)

## -----------------------------------------------------------------------------
# load data
file_counts <- system.file("extdata/vignette_counts.txt", package = "regsplice")
data <- read.table(file_counts, header = TRUE, sep = "\t", stringsAsFactors = FALSE)

head(data)

# extract counts, gene_IDs, and n_exons
counts <- data[, 2:7]
tbl_exons <- table(sapply(strsplit(data$exon, ":"), function(s) s[[1]]))
gene_IDs <- names(tbl_exons)
n_exons <- unname(tbl_exons)

dim(counts)

length(gene_IDs)

head(gene_IDs)

length(n_exons)

sum(n_exons)

# create condition vector
condition <- rep(c("untreated", "treated"), each = 3)

condition

## -----------------------------------------------------------------------------
library(regsplice)

rs_data <- RegspliceData(counts, gene_IDs, n_exons, condition)

rs_results <- suppressWarnings(regsplice(rs_data, seed = 123))

## -----------------------------------------------------------------------------
summaryTable(rs_results)

## -----------------------------------------------------------------------------
library(regsplice)

rs_data <- RegspliceData(counts, gene_IDs, n_exons, condition)

## -----------------------------------------------------------------------------
rs_data <- filterZeros(rs_data)

## -----------------------------------------------------------------------------
rs_data <- filterLowCounts(rs_data)

## -----------------------------------------------------------------------------
rs_data <- runNormalization(rs_data)

## -----------------------------------------------------------------------------
rs_data <- runVoom(rs_data)

# view column meta-data including normalization factors and normalized library sizes
colData(rs_data)

## -----------------------------------------------------------------------------
rs_results <- initializeResults(rs_data)

## -----------------------------------------------------------------------------
# set random seed for reproducibility
seed <- 123

# fit regularized models
rs_results <- suppressWarnings(fitRegMultiple(rs_results, rs_data, seed = seed))

# fit null models
rs_results <- fitNullMultiple(rs_results, rs_data, seed = seed)

# fit "full" models (not required if 'when_null_selected = "ones"' in next step)
rs_results <- fitFullMultiple(rs_results, rs_data, seed = seed)

## -----------------------------------------------------------------------------
rs_results <- LRTests(rs_results)

## -----------------------------------------------------------------------------
summaryTable(rs_results)

## -----------------------------------------------------------------------------
summaryTable(rs_results, n = Inf)

## -----------------------------------------------------------------------------
sum(p_adj(rs_results) < 0.05)

table(p_adj(rs_results) < 0.05)

## -----------------------------------------------------------------------------
# load true DS status labels
file_truth <- system.file("extdata/vignette_truth.txt", package = "regsplice")
data_truth <- read.table(file_truth, header = TRUE, sep = "\t", stringsAsFactors = FALSE)

str(data_truth)

# remove genes that were filtered during regsplice analysis
data_truth <- data_truth[data_truth$gene %in% gene_IDs(rs_results), ]

dim(data_truth)

length(gene_IDs(rs_results))

# number of true DS genes in simulated data set
sum(data_truth$ds_status == 1)

table(data_truth$ds_status)

# contingency table comparing true and predicted DS status for each gene
# (significance threshold: FDR < 0.05)
table(true = data_truth$ds_status, predicted = p_adj(rs_results) < 0.05)

# increasing the threshold detects more genes, at the expense of more false positives
table(true = data_truth$ds_status, predicted = p_adj(rs_results) < 0.99)

## -----------------------------------------------------------------------------
# gene with 3 exons
# 4 biological samples; 2 samples in each of 2 conditions
design_example <- createDesignMatrix(condition = rep(c(0, 1), each = 2), n_exons = 3)

design_example

###########################
### S4 class definition ###
###########################


#' @rdname RegspliceResults
#' @export
#' 
setClass("RegspliceResults", 
         slots = list(gene_IDs = "character", 
                      fit_reg_dev = "numeric", fit_reg_df = "numeric", 
                      fit_null_dev = "numeric", fit_null_df = "numeric", 
                      fit_full_dev = "numeric", fit_full_df = "numeric", 
                      p_vals = "numeric", p_adj = "numeric", 
                      LR_stats = "numeric", df_tests = "numeric"))




############################
### Constructor function ###
############################


#' RegspliceResults objects.
#' 
#' \code{RegspliceResults} objects contain the results of a \code{regsplice} analysis.
#' 
#' \code{RegspliceResults} objects are created with the constructor function
#' \code{RegspliceResults()}, which requires the gene IDs as an argument.
#' 
#' Once created, \code{RegspliceResults} objects are then populated using the functions 
#' \code{\link{fitRegMultiple}}, \code{\link{fitNullMultiple}},
#' \code{\link{fitFullMultiple}}, and \code{\link{LRTests}}.
#' 
#' The function \code{\link{summaryTable}} can be used to display a summary table of the
#' results.
#' 
#' 
#' @param gene_IDs Gene identifiers or names (character vector).
#' @param x \code{RegspliceResults} object (for accessor functions).
#' 
#' 
#' @field gene_IDs Gene identifiers or names (character vector).
#' @field fit_reg_dev Deviance of fitted regularized (lasso) models from 
#'   \code{\link{fitRegMultiple}}.
#' @field fit_reg_df Degrees of freedom of fitted regularized (lasso) models from 
#'   \code{\link{fitRegMultiple}}.
#' @field fit_null_dev Deviance of fitted null models from 
#'   \code{\link{fitNullMultiple}}.
#' @field fit_null_df Degrees of freedom of fitted null models from 
#'   \code{\link{fitNullMultiple}}.
#' @field fit_full_dev Deviance of fitted full models from 
#'   \code{\link{fitFullMultiple}}.
#' @field fit_full_df Degrees of freedom of fitted full models from 
#'   \code{\link{fitFullMultiple}}.
#' @field p_vals Raw p-values (numeric vector).
#' @field p_adj Multiple testing adjusted p-values (Benjamini-Hochberg false discovery 
#'   rates, FDR).
#' @field LR_stats Likelihood ratio test statistics.
#' @field df_tests Degrees of freedom of likelihood ratio tests.
#' 
#' 
#' @section Accessor functions:
#' 
#' \itemize{
#' \item \code{gene_IDs()}: Accesses gene identifiers or names.
#' \item \code{p_vals()}: Accesses raw p-values.
#' \item \code{p_adj()}: Accesses multiple testing adjusted p-values (Benjamini-Hochberg false discovery rates, FDR).
#' \item \code{LR_stats()}: Accesses likelihood ratio test statistics.
#' \item \code{df_tests()}: Accesses degrees of freedom of likelihood ratio tests.
#' }
#' 
#' 
#' @return Returns an empty \code{RegspliceResults} object.
#' 
#' @seealso \code{\link{fitRegMultiple}} \code{\link{fitNullMultiple}}
#'   \code{\link{fitFullMultiple}} \code{\link{LRTests}} \code{\link{summaryTable}}
#' 
#' @importFrom methods new
#' 
#' @export
#' 
#' @examples
#' # initialize RegspliceResults object
#' gene_IDs <- paste0("gene", 1:5)
#' RegspliceResults(gene_IDs)
#' 
RegspliceResults <- function(gene_IDs) {
  
  new("RegspliceResults", gene_IDs = gene_IDs)
  
}




##########################
### Accessor functions ###
##########################


#' @rdname RegspliceResults
#' @export
#' 
setGeneric("gene_IDs", function(x) {
  standardGeneric("gene_IDs")
})


#' @rdname RegspliceResults
#' @export
#' 
setMethod("gene_IDs", "RegspliceResults", function(x) {
  x@gene_IDs
})




#' @rdname RegspliceResults
#' @export
#' 
setGeneric("p_vals", function(x) {
  standardGeneric("p_vals")
})


#' @rdname RegspliceResults
#' @export
#' 
setMethod("p_vals", "RegspliceResults", function(x) {
  x@p_vals
})




#' @rdname RegspliceResults
#' @export
#' 
setGeneric("p_adj", function(x) {
  standardGeneric("p_adj")
})


#' @rdname RegspliceResults
#' @export
#' 
setMethod("p_adj", "RegspliceResults", function(x) {
  x@p_adj
})




#' @rdname RegspliceResults
#' @export
#' 
setGeneric("LR_stats", function(x) {
  standardGeneric("LR_stats")
})


#' @rdname RegspliceResults
#' @export
#' 
setMethod("LR_stats", "RegspliceResults", function(x) {
  x@LR_stats
})




#' @rdname RegspliceResults
#' @export
#' 
setGeneric("df_tests", function(x) {
  standardGeneric("df_tests")
})


#' @rdname RegspliceResults
#' @export
#' 
setMethod("df_tests", "RegspliceResults", function(x) {
  x@df_tests
})

#' @include class_RegspliceData.R class_RegspliceResults.R
NULL




#' Calculate normalization factors.
#' 
#' Calculate normalization factors to scale library sizes, using the TMM (trimmed mean of
#' M-values) method implemented in \code{edgeR}.
#' 
#' Normalization factors are used to scale the raw library sizes (total read counts per 
#' sample). We use the TMM (trimmed mean of M-values) normalization method (Robinson and 
#' Oshlack, 2010), as implemented in the \code{edgeR} package.
#' 
#' For more details, see the documentation for \code{\link[edgeR]{calcNormFactors}} in 
#' the \code{edgeR} package.
#' 
#' This step should be performed after filtering with \code{\link{filterZeros}} and 
#' \code{\link{filterLowCounts}}. The normalization factors are then used by 
#' \code{limma-voom} in the next step (\code{\link{runVoom}}).
#' 
#' The normalization factors are stored in a new column named \code{norm_factors} in the 
#' column meta-data (\code{colData} slot) of the \code{\linkS4class{RegspliceData}} 
#' object. The \code{colData} can be accessed with the accessor function 
#' \code{colData()}.
#' 
#' Normalization should be skipped when using exon microarray data. (When using the 
#' \code{\link{regsplice}} wrapper function, normalization can be disabled with the 
#' argument \code{normalize = FALSE}).
#' 
#' Previous step: Filter low-count exon bins with \code{\link{filterLowCounts}}.
#' Next step: Calculate \code{limma-voom} transformation and weights with 
#' \code{\link{runVoom}}.
#' 
#' 
#' @param rs_data \code{\linkS4class{RegspliceData}} object, which has already been 
#'   filtered with \code{\link{filterZeros}} and \code{\link{filterLowCounts}}.
#' @param norm_method Normalization method to use. Options are \code{"TMM"}, 
#'   \code{"RLE"}, \code{"upperquartile"}, and \code{"none"}. See documentation for 
#'   \code{\link[edgeR]{calcNormFactors}} in \code{edgeR} package for details. Default is
#'   \code{"TMM"}.
#' 
#' 
#' @return Returns a \code{\linkS4class{RegspliceData}} object. Normalization factors are
#'   stored in the column \code{norm_factors} in the column meta-data (\code{colData} 
#'   slot), which can be accessed with the \code{colData()} accessor function.
#' 
#' @seealso \code{\link{filterLowCounts}} \code{\link{runVoom}}
#' 
#' @importFrom edgeR calcNormFactors
#' @importFrom S4Vectors DataFrame SimpleList
#' 
#' @export
#' 
#' @examples 
#' file_counts <- system.file("extdata/vignette_counts.txt", package = "regsplice")
#' data <- read.table(file_counts, header = TRUE, sep = "\t", stringsAsFactors = FALSE)
#' head(data)
#' 
#' counts <- data[, 2:7]
#' tbl_exons <- table(sapply(strsplit(data$exon, ":"), function(s) s[[1]]))
#' gene_IDs <- names(tbl_exons)
#' n_exons <- unname(tbl_exons)
#' condition <- rep(c("untreated", "treated"), each = 3)
#' 
#' rs_data <- RegspliceData(counts, gene_IDs, n_exons, condition)
#' 
#' rs_data <- filterZeros(rs_data)
#' rs_data <- filterLowCounts(rs_data)
#' rs_data <- runNormalization(rs_data)
#' 
runNormalization <- function(rs_data, norm_method = "TMM") {
  
  norm_method <- match.arg(norm_method, c("TMM", "RLE", "upperquartile", "none"))
  
  counts <- countsData(rs_data)
  norm_factors <- edgeR::calcNormFactors(counts, method = norm_method)
  
  colData(rs_data)$norm_factors <- norm_factors
  
  rs_data
}

#' @include class_RegspliceData.R class_RegspliceResults.R
NULL




#' Initialize RegspliceResults object.
#' 
#' Initialize a \code{RegspliceResults} object, which will contain the results of the 
#' \code{regsplice} analysis.
#' 
#' Creates a \code{\linkS4class{RegspliceResults}} object containing gene names only. 
#' This object will subsequently be populated using the functions 
#' \code{\link{fitRegMultiple}}, \code{\link{fitNullMultiple}}, 
#' \code{\link{fitFullMultiple}}, and \code{\link{LRTests}}.
#' 
#' Previous step: Calculate \code{limma-voom} transformation and weights with 
#' \code{\link{runVoom}}.
#' Next step: Fit models with \code{\link{fitRegMultiple}}, 
#' \code{\link{fitNullMultiple}}, and \code{\link{fitFullMultiple}}.
#' 
#' 
#' @param rs_data \code{\linkS4class{RegspliceData}} object. This should contain gene IDs
#'   in a column named \code{gene_IDs} in the row meta-data, which can be accessed with 
#'   the accessor function \code{\link{rowData}}.
#' 
#' 
#' @return Returns a \code{\linkS4class{RegspliceResults}} object containing gene IDs 
#'   only.
#' 
#' @seealso \code{\linkS4class{RegspliceData}} \code{\linkS4class{RegspliceResults}} 
#'   \code{\link{fitRegMultiple}} \code{\link{fitNullMultiple}} 
#'   \code{\link{fitFullMultiple}} \code{\link{LRTests}} \code{\link{summaryTable}}
#' 
#' @export
#' 
#' @examples
#' file_counts <- system.file("extdata/vignette_counts.txt", package = "regsplice")
#' data <- read.table(file_counts, header = TRUE, sep = "\t", stringsAsFactors = FALSE)
#' head(data)
#' 
#' counts <- data[, 2:7]
#' tbl_exons <- table(sapply(strsplit(data$exon, ":"), function(s) s[[1]]))
#' gene_IDs <- names(tbl_exons)
#' n_exons <- unname(tbl_exons)
#' condition <- rep(c("untreated", "treated"), each = 3)
#' 
#' rs_data <- RegspliceData(counts, gene_IDs, n_exons, condition)
#' 
#' rs_data <- filterZeros(rs_data)
#' rs_data <- filterLowCounts(rs_data)
#' rs_data <- runNormalization(rs_data)
#' rs_data <- runVoom(rs_data)
#' 
#' rs_results <- initializeResults(rs_data)
#' 
initializeResults <- function(rs_data) {
  
  # unique gene IDs in alphabetical order
  gene_IDs <- names(table(rowData(rs_data)$gene_IDs))
  
  RegspliceResults(gene_IDs)
  
}

#' @include class_RegspliceData.R class_RegspliceResults.R
NULL




#' Fit models.
#' 
#' Model fitting functions for \code{regsplice} package.
#' 
#' There are three model fitting functions:
#' 
#' \code{fitRegMultiple} fits regularized (lasso) models containing an optimal subset of 
#' exon:condition interaction terms for each gene. The model fitting procedure penalizes 
#' the interaction terms only, so that the main effect terms for exons and samples are 
#' always included. This ensures that the null model is nested, allowing likelihood ratio
#' tests to be calculated.
#' 
#' \code{fitNullMultiple} fits the null models, which do not contain any interaction 
#' terms.
#' 
#' \code{fitFullMultiple} fits full models, which contain all exon:condition interaction
#' terms for each gene.
#' 
#' See \code{\link{createDesignMatrix}} for more details about the terms in each model.
#' 
#' The fitting functions fit models for all genes in the data set.
#' 
#' A random seed can be provided with the \code{seed} argument, to generate reproducible
#' results.
#' 
#' If the \code{rs_data} object does not contain a weights matrix, all exon bins are 
#' weighted equally.
#' 
#' Previous step: Initialize \code{\linkS4class{RegspliceResults}} object with 
#' \code{\link{initializeResults}}.
#' Next step: Calculate likelihood ratio tests with \code{\link{LRTests}}.
#' 
#' 
#' @param rs_results \code{\linkS4class{RegspliceResults}} object, which will be used to 
#'   store results. Initialized using the constructor function \code{RegspliceResults()}.
#'   See \code{\linkS4class{RegspliceResults}} for details.
#' @param rs_data \code{\linkS4class{RegspliceData}} object. In the case of RNA-seq read 
#'   count data, this has been pre-transformed with \code{\link{runVoom}}. Contains 
#'   \code{counts} and \code{weights} data matrices, and vector of experimental 
#'   conditions for each biological sample stored in \code{colData}. See 
#'   \code{\linkS4class{RegspliceData}} for details.
#' @param alpha Elastic net parameter \code{alpha} for \code{glmnet} model fitting 
#'   functions. Must be between 0 (ridge regression) and 1 (lasso). Default is 1 (lasso).
#'   See \code{glmnet} documentation for more details.
#' @param lambda_choice Parameter to select which optimal \code{lambda} value to choose 
#'   from the \code{cv.glmnet} cross validation fit. Choices are "lambda.min" (model with
#'   minimum cross-validated error) and "lambda.1se" (most regularized model with 
#'   cross-validated error within one standard error of minimum). Default is 
#'   "lambda.min". See \code{glmnet} documentation for more details.
#' @param seed Random seed (integer). Default is NULL. Provide an integer value to set 
#'   the random seed for reproducible results.
#' @param ... Other arguments to pass to \code{cv.glmnet}, \code{glmnet}, or \code{glm}.
#' 
#'   
#' @return Returns a \code{\linkS4class{RegspliceResults}} object containing deviance and
#'   degrees of freedom of the fitted models. See \code{\linkS4class{RegspliceResults}}
#'   for details.
#' 
#' 
#' @seealso \code{\link{createDesignMatrix}} \code{\link{RegspliceResults}} 
#'   \code{\link{initializeResults}} \code{\link{LRTests}}
#' 
#' @seealso \code{\link[glmnet]{glmnet}} \code{\link[glmnet]{cv.glmnet}} 
#'   \code{\link[stats]{glm}}
#'   
#' @importFrom pbapply pblapply pboptions
#' 
#' @export
#' 
#' @examples
#' file_counts <- system.file("extdata/vignette_counts.txt", package = "regsplice")
#' data <- read.table(file_counts, header = TRUE, sep = "\t", stringsAsFactors = FALSE)
#' head(data)
#' 
#' counts <- data[, 2:7]
#' tbl_exons <- table(sapply(strsplit(data$exon, ":"), function(s) s[[1]]))
#' gene_IDs <- names(tbl_exons)
#' n_exons <- unname(tbl_exons)
#' condition <- rep(c("untreated", "treated"), each = 3)
#' 
#' rs_data <- RegspliceData(counts, gene_IDs, n_exons, condition)
#' 
#' rs_data <- filterZeros(rs_data)
#' rs_data <- filterLowCounts(rs_data)
#' rs_data <- runNormalization(rs_data)
#' rs_data <- runVoom(rs_data)
#' 
#' rs_results <- initializeResults(rs_data)
#' 
#' rs_results <- fitRegMultiple(rs_results, rs_data)
#' rs_results <- fitNullMultiple(rs_results, rs_data)
#' rs_results <- fitFullMultiple(rs_results, rs_data)
#' 
fitRegMultiple <- function(rs_results, rs_data, 
                           alpha = 1, lambda_choice = c("lambda.min", "lambda.1se"), 
                           seed = NULL, ...) {
  
  lambda_choice <- match.arg(lambda_choice)
  
  gene_IDs <- names(table(rowData(rs_data)$gene_IDs))
  n_genes <- length(gene_IDs)
  
  FUN <- function(i) {
    gene_ID_i <- gene_IDs[i]
    if (gene_ID_i != rs_results@gene_IDs[i]) stop("gene IDs do not match")
    
    rs_data_i <- rs_data[gene_ID_i, ]
    
    .fitRegSingle(rs_data = rs_data_i, alpha = alpha, lambda_choice = lambda_choice, ...)
  }
  
  message("Fitting regularized (lasso) models...")
  
  if (is.null(seed)) seed <- as.numeric(Sys.time())
  
  # options for progress bar
  pbapply::pboptions(char = "=")
  
  set.seed(seed)
  out <- pbapply::pblapply(seq_len(n_genes), FUN = FUN)
  
  # collapse lists
  dev_collapse <- sapply(out, "[[", "dev")
  df_collapse <- sapply(out, "[[", "df")
  
  rs_results@fit_reg_dev <- dev_collapse
  rs_results@fit_reg_df <- df_collapse
  
  rs_results
}



#' @rdname fitRegMultiple
#' @importFrom pbapply pblapply pboptions
#' @export
#' 
fitNullMultiple <- function(rs_results, rs_data, seed = NULL, ...) {
  
  gene_IDs <- names(table(rowData(rs_data)$gene_IDs))
  n_genes <- length(gene_IDs)
  
  FUN <- function(i) {
    gene_ID_i <- gene_IDs[i]
    if (gene_ID_i != rs_results@gene_IDs[i]) stop("gene IDs do not match")
    
    rs_data_i <- rs_data[gene_ID_i, ]
    
    .fitNullSingle(rs_data_i, ...)
  }
  
  message("Fitting null models...")
  
  if (is.null(seed)) seed <- as.numeric(Sys.time())
  
  # options for progress bar
  pbapply::pboptions(char = "=")
  
  set.seed(seed)
  out <- pbapply::pblapply(seq_len(n_genes), FUN = FUN)
  
  # collapse lists
  dev_collapse <- sapply(out, "[[", "dev")
  df_collapse <- sapply(out, "[[", "df")
  
  rs_results@fit_null_dev <- dev_collapse
  rs_results@fit_null_df <- df_collapse
  
  rs_results
}



#' @rdname fitRegMultiple
#' @importFrom pbapply pblapply pboptions
#' @export
#' 
fitFullMultiple <- function(rs_results, rs_data, seed = NULL, ...) {
  
  gene_IDs <- names(table(rowData(rs_data)$gene_IDs))
  n_genes <- length(gene_IDs)
  
  FUN <- function(i) {
    gene_ID_i <- gene_IDs[i]
    if (gene_ID_i != rs_results@gene_IDs[i]) stop("gene IDs do not match")
    
    rs_data_i <- rs_data[gene_ID_i, ]
    
    .fitFullSingle(rs_data_i, ...)
  }
  
  message("Fitting full models...")
  
  if (is.null(seed)) seed <- as.numeric(Sys.time())
  
  # options for progress bar
  pbapply::pboptions(char = "=")
  
  set.seed(seed)
  out <- pbapply::pblapply(seq_len(n_genes), FUN = FUN)
  
  # collapse lists
  dev_collapse <- sapply(out, "[[", "dev")
  df_collapse <- sapply(out, "[[", "df")
  
  rs_results@fit_full_dev <- dev_collapse
  rs_results@fit_full_df <- df_collapse
  
  rs_results
}

#' @include class_RegspliceData.R class_RegspliceResults.R
NULL




#' Filter low-count exons.
#' 
#' Filter low-count exons from RNA-seq read count data.
#' 
#' Filters low-count exon bins from RNA-seq read count data. Any remaining single-exon 
#' genes (after filtering) are also removed (since differential splicing requires
#' multiple exon bins).
#' 
#' Input data is assumed to be in the form of a \code{RegspliceData} object. See 
#' \code{\link{RegspliceData}} for details.
#' 
#' The arguments \code{filter_min_per_exon} and \code{filter_min_per_sample} control the 
#' amount of filtering. Exon bins that meet the filtering conditions are kept. Default 
#' values for the arguments are provided; however, these should be adjusted depending on 
#' the total number of samples and the number of samples per condition.
#' 
#' After filtering low-count exon bins, any remaining genes containing only a single exon
#' bin are also removed (since differential splicing requires multiple exon bins).
#' 
#' Filtering should be skipped when using exon microarray data. (When using the 
#' \code{regsplice} wrapper function, filtering can be disabled with the argument 
#' \code{filter = FALSE}).
#' 
#' Previous step: Filter zero-count exon bins with \code{\link{filterZeros}}.
#' Next step: Calculate normalization factors with \code{\link{runNormalization}}.
#' 
#' 
#' @param rs_data \code{\linkS4class{RegspliceData}} object.
#' @param filter_min_per_exon Filtering parameter: minimum number of reads per exon bin, 
#'   summed across all biological samples. Default is 6.
#' @param filter_min_per_sample Filtering parameter: minimum number of reads per 
#'   biological sample; i.e. for each exon bin, at least one sample must have this number
#'   of reads. Default is 3.
#' 
#' 
#' @return Returns a \code{\linkS4class{RegspliceData}} object.
#' 
#' @seealso \code{\link{filterZeros}} \code{\link{runNormalization}}
#' 
#' @importFrom methods is
#' 
#' @export
#' 
#' @examples 
#' file_counts <- system.file("extdata/vignette_counts.txt", package = "regsplice")
#' data <- read.table(file_counts, header = TRUE, sep = "\t", stringsAsFactors = FALSE)
#' head(data)
#' 
#' counts <- data[, 2:7]
#' tbl_exons <- table(sapply(strsplit(data$exon, ":"), function(s) s[[1]]))
#' gene_IDs <- names(tbl_exons)
#' n_exons <- unname(tbl_exons)
#' condition <- rep(c("untreated", "treated"), each = 3)
#' 
#' rs_data <- RegspliceData(counts, gene_IDs, n_exons, condition)
#' 
#' rs_data <- filterZeros(rs_data)
#' rs_data <- filterLowCounts(rs_data)
#' 
filterLowCounts <- function(rs_data, filter_min_per_exon = 6, filter_min_per_sample = 3) {
  
  if (!("RegspliceData" %in% is(rs_data))) stop("'rs_data' must be a 'RegspliceData' object")
  
  counts <- countsData(rs_data)
  filt_per_exon <- rowSums(counts) >= filter_min_per_exon
  filt_per_sample <- apply(counts, MARGIN = 1, FUN = function(r) max(r) >= filter_min_per_sample)
  
  ix_keep <- filt_per_exon & filt_per_sample
  
  message(paste("removed", sum(!ix_keep), "low-count exon(s)"))
  
  rs_data <- suppressMessages(rs_data[ix_keep, ])
  
  # remove any remaining single-exon genes after filtering
  .removeSingleExonGenes(rs_data)
}

---
title: "regsplice workflow"
author: 
  - name: Lukas M. Weber
    affiliation: 
      - &id1 "Institute of Molecular Life Sciences, University of Zurich, Zurich, Switzerland"
      - &id2 "SIB Swiss Institute of Bioinformatics, Zurich, Switzerland"
  - name: Mark D. Robinson
    affiliation: 
      - *id1
      - *id2
package: regsplice
output: 
  BiocStyle::html_document
vignette: >
  %\VignetteIndexEntry{regsplice workflow}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(collapse = TRUE)
```


# Introduction

The `regsplice` package implements statistical methods for the detection of differential exon usage (differential splicing) in RNA sequencing (RNA-seq) and exon microarray data sets.

The `regsplice` methods are based on the use of the lasso (L1-regularization) to improve the power of standard generalized linear models. A key advantage of `regsplice` is that runtimes are fast compared to other leading approaches. We anticipate that similar regularization-based methods may also have applications in other settings.

The detailed statistical methodology and performance comparisons with other methods will be described in an upcoming paper.



## Example workflow

This vignette demonstrates an example workflow for the `regsplice` package, using a small simulated RNA-seq data set.

There are two options for running `regsplice`: you can run a complete workflow in one step using the wrapper function `regsplice()`; or you can run the individual functions for each step in sequence, which provides additional flexibility and insight into the methodology. Both options are demonstrated below.



## Data set

The data set used for the example workflow consists of exon-level read counts for a subset of 100 genes from a simulated human RNA-seq data set, consisting of 6 biological samples, with 3 samples in each of 2 conditions.

The original data set is from the paper:

> Soneson, Matthes et al. (2016), *Isoform prefiltering improves performance of count-based methods for analysis of differential transcript usage*, Genome Biology, [available here](https://genomebiology.biomedcentral.com/articles/10.1186/s13059-015-0862-3)

Original data files from this paper, containing the simulated RNA-seq reads (FASTQ and BAM files), are available from ArrayExpress at accession code [E-MTAB-3766](http://www.ebi.ac.uk/arrayexpress/experiments/E-MTAB-3766/).

Exon bin counts were generated with the Python counting scripts provided with the [DEXSeq](http://bioconductor.org/packages/release/bioc/html/DEXSeq.html) package, using the option to exclude exons from overlapping genes instead of aggregating them into multi-gene complexes (see Soneson et al. 2016, Supplementary Material).

For this example workflow, we have selected a subset consisting of the first 100 genes from this simulated data set. The exon-level read counts and the true differential splicing status labels for these 100 genes are saved in the text files `vignette_counts.txt` and `vignette_truth.txt` in the `extdata/` directory in the `regsplice` package source code.



## Exon microarray data

The `regsplice` methods are designed to work with both RNA-seq read counts and exon microarray intensities.

If you are using exon microarray data, the main steps in the workflow are the same as shown below for RNA-seq data. However, the following adjustments to the workflow are required:

- Instead of RNA-seq read counts, a matrix or data frame of exon microarray intensities is provided to the `counts` input argument. The name of the argument is still `counts`, regardless of the input data type.

- Exon microarray intensities should be log2-transformed externally, before they are provided to `regsplice`. This is usually done during pre-processing of microarray data, and may be done automatically depending on your software.

- Filtering of zero-count and low-count exon bins should be disabled, by setting the arguments `filter_zeros = FALSE` and `filter_low_counts = FALSE`.

- Calculation of normalization factors should be disabled, by setting `normalize = FALSE`.

- Calculation of `limma-voom` transformation and weights should be disabled, by setting `voom = FALSE`.




# Workflow

## Load data and create condition vector

Load the vignette example data file, which contains simulated RNA-seq read counts for 100 genes across 6 biological samples. From the raw data, extract the table of counts (`counts`), gene IDs (`gene_IDs`), and number of exon bins per gene (`n_exons`).

Then create the `condition` vector, which specifies the experimental conditions or treatment groups for each biological sample.

\vspace{6pt}

```{r}
# load data
file_counts <- system.file("extdata/vignette_counts.txt", package = "regsplice")
data <- read.table(file_counts, header = TRUE, sep = "\t", stringsAsFactors = FALSE)

head(data)

# extract counts, gene_IDs, and n_exons
counts <- data[, 2:7]
tbl_exons <- table(sapply(strsplit(data$exon, ":"), function(s) s[[1]]))
gene_IDs <- names(tbl_exons)
n_exons <- unname(tbl_exons)

dim(counts)

length(gene_IDs)

head(gene_IDs)

length(n_exons)

sum(n_exons)

# create condition vector
condition <- rep(c("untreated", "treated"), each = 3)

condition
```



## Run workflow with wrapper function

The `regsplice()` wrapper function runs the analysis pipeline with a single command. The required input format for the wrapper function is a `RegspliceData` object, which is created with the `RegspliceData()` constructor function.

The results of a `regsplice` analysis consist of a set of multiple testing adjusted p-values (Benjamini-Hochberg false discovery rates, FDR) quantifying the statistical evidence for differential exon usage (DEU) for each gene. The adjusted p-values are used to rank the genes in the data set according to their evidence for DEU, and a significance threshold can be specified to generate a list of genes with statistically significant evidence for DEU.

The wrapper function returns gene names, fitted model results, raw p-values, multiple testing adjusted p-values, likelihood ratio (LR) test statistics, and degrees of freedom of the LR tests.

The required inputs for the `RegspliceData()` constructor function are `counts` (matrix or data frame of RNA-seq read counts or exon microarray intensities), `gene_IDs` (vector of gene IDs), `n_exons` (vector of exon lengths, i.e. number of exon bins per gene), and `condition` (vector of experimental conditions for each biological sample).

Alternatively, the inputs can be provided as a `SummarizedExperiment` object, which will be parsed to extract these components. This may be useful when running `regsplice` as part of a pipeline with other packages.

Note that we have used `suppressWarnings()` to hide warning messages related to the small number of observations per gene in this example data set. For most data sets, these warning messages should not be present.

See `?RegspliceData` and `?regsplice` for additional details, including other available inputs and options. The `seed` argument is used to generate reproducible results.

\vspace{6pt}

```{r}
library(regsplice)

rs_data <- RegspliceData(counts, gene_IDs, n_exons, condition)

rs_results <- suppressWarnings(regsplice(rs_data, seed = 123))
```



### Summary table of results

The function `summaryTable()` is used to generate a summary table of the results.

The results are displayed as a data frame of the top `n` most highly significant genes, ranked according to either the false discovery rate (FDR) or raw p-values, up to a specified significance threshold (e.g. FDR < 0.05).

The argument `rank_by` chooses whether to rank by FDR or raw p-values.

To display results for all genes up to the significance threshold, set the argument `n = Inf`. To display results for all genes in the data set, set both `n = Inf` and `threshold = 1`.

For more details, see `?summaryTable`.

\vspace{6pt}

```{r}
summaryTable(rs_results)
```




## Run workflow using functions for individual steps

Alternatively, the `regsplice` analysis pipeline can be run using the individual functions for each step, which provides additional flexibility and insight into the statistical methodology. The steps are described below.



### Create RegspliceData object

The first step is to create a `RegspliceData` object, which contains data in the format required by the functions in the `regsplice` analysis pipeline.

The `RegspliceData` format is based on the `SummarizedExperiment` container from Bioconductor. The main advantage of this format is that subsetting operations automatically keep data and meta-data for rows and columns in sync, which helps avoid errors caused by selecting incorrect row or column indices.

Initially, the `RegspliceData` objects contain raw data along with meta-data for rows (genes and exon bins) and columns (biological samples). During subsequent steps in the `regsplice` analysis pipeline, the data values are modified, and additional data and meta-data are added. Final results are stored in a `RegspliceResults` object.

The required inputs are `counts` (matrix or data frame of RNA-seq read counts or exon microarray intensities), `gene_IDs` (vector of gene IDs), `n_exons` (vector of exon lengths, i.e. number of exon bins per gene), and `condition` (vector of experimental conditions for each biological sample).

Alternatively, the inputs can be provided as a `SummarizedExperiment` object, which will be parsed to extract these components. This may be useful when running `regsplice` as part of a pipeline with other packages.

Note that the warning messages are due to the small number of observations per gene in the data set used in this vignette. In most data sets, these warning messages should not be present.

For more details, see `?RegspliceData`.

\vspace{6pt}

```{r}
library(regsplice)

rs_data <- RegspliceData(counts, gene_IDs, n_exons, condition)
```



### Filter zero-count exon bins

Next, use the function `filterZeros()` to filter exon bins (rows) with zero counts in all biological samples (columns).

Any remaining single-exon genes are also removed (since differential splicing requires multiple exons).

If you are using exon microarray data, this step should be skipped.

For more details, see `?filterZeros`.

\vspace{6pt}

```{r}
rs_data <- filterZeros(rs_data)
```



### Filter low-count exons

Filter low-count exon bins with `filterLowCounts()`.

The arguments `filter_min_per_exon` and `filter_min_per_sample` control the amount of filtering. Default values are provided; however, these should be adjusted depending on the total number of samples and the number of samples per condition.

Any remaining single-exon genes are also removed.

If you are using exon microarray data, this step should be skipped.

For more details, see `?filterLowCounts`.

\vspace{6pt}

```{r}
rs_data <- filterLowCounts(rs_data)
```



### Calculate normalization factors

The function `runNormalization()` calculates normalization factors, which are used to scale library sizes.

By default, `runNormalization()` uses the TMM (trimmed mean of M-values) normalization method (Robinson and Oshlack, 2010), implemented in the `edgeR` package. For more details, see the documentation for `calcNormFactors()` in the `edgeR` package.

This step should be done after filtering. The normalization factors are then used by `limma-voom` in the next step.

If you are using exon microarray data, this step should be skipped.

For more details, see `?runNormalization`.

```{r}
rs_data <- runNormalization(rs_data)
```



### 'voom' transformation and weights

The next step is to use `limma-voom` to transform the counts and calculate exon-level weights. This is done with the `runVoom()` function.

The `limma-voom` methodology transforms counts to log2-counts per million (logCPM), and calculates exon-level weights based on the observed mean-variance relationship. This is required because raw or log-transformed counts do not fulfill the statistical assumptions required for linear modeling (i.e. equal variance). After the `limma-voom` transformation and weights have been calculated, linear modeling methods can be used.

For more details, see the following paper, which introduced `voom`; or the [limma User's Guide](http://bioconductor.org/packages/release/bioc/html/limma.html) (section "Differential splicing") available on Bioconductor.

- Law et al. (2014), *voom: precision weights unlock linear model analysis tools for RNA-seq read counts*, Genome Biology, [available here](http://genomebiology.biomedcentral.com/articles/10.1186/gb-2014-15-2-r29)

Note that `voom` assumes that exon bins (rows) with zero or low counts have already been removed, so this step should be done after filtering with `filterZeros()` and `filterLowCounts()`.

If normalization factors are available (from previous step with `runNormalization()`), they will be used by `voom` to calculate normalized library sizes. If they are not available, `voom` will use non-normalized library sizes (columnwise total counts) instead.

If you are using exon microarray data, this step should be skipped.

For more details, see `?runVoom`.

\vspace{6pt}

```{r}
rs_data <- runVoom(rs_data)

# view column meta-data including normalization factors and normalized library sizes
colData(rs_data)
```



### Initialize RegspliceResults object

The `initializeResults()` function creates a `RegspliceResults` object, which will contain the results of the analysis. This object will be populated in the subsequent steps.

For more details, see `?initializeResults`.

\vspace{6pt}

```{r}
rs_results <- initializeResults(rs_data)
```



### Fit models

There are three model fitting functions: 

- `fitRegMultiple()` fits regularized (lasso) models containing an optimal subset of exon:condition interaction terms for each gene. The model fitting procedure penalizes the interaction terms only, so that the main effect terms for exons and samples are always included. This ensures that the null model is nested, allowing likelihood ratio tests to be calculated.

- `fitNullMultiple()` fits the null models, which do not contain any interaction terms.

- `fitFullMultiple()` fits "full" models, which contain all exon:condition interaction terms for each gene.

The fitting functions fit models for all genes in the data set.

Note that we have used `suppressWarnings()` to hide warning messages related to the small number of observations per gene in this example data set. For most data sets, these warning messages should not be present.

For more details, see `?fitRegMultiple`, `?fitNullMultiple`, or `?fitFullMultiple`.

\vspace{6pt}

```{r}
# set random seed for reproducibility
seed <- 123

# fit regularized models
rs_results <- suppressWarnings(fitRegMultiple(rs_results, rs_data, seed = seed))

# fit null models
rs_results <- fitNullMultiple(rs_results, rs_data, seed = seed)

# fit "full" models (not required if 'when_null_selected = "ones"' in next step)
rs_results <- fitFullMultiple(rs_results, rs_data, seed = seed)
```



### Calculate likelihood ratio tests

The function `LRTests()` calculates likelihood ratio (LR) tests between the fitted models and null models.

If the fitted regularized (lasso) model contains at least one exon:condition interaction term, the LR test compares the lasso model against the nested null model. However, if the lasso model contains zero interaction terms, then the lasso and null models are identical, so the LR test cannot be calculated. The `when_null_selected` argument lets the user choose what to do in these cases: either set p-values equal to 1 (`when_null_selected = "ones"`); or calculate a LR test using the "full" model containing all exon:condition interaction terms (`when_null_selected = "full"`), which reduces power due to the larger number of terms, but allows the evidence for differential exon usage among these genes to be distinguished. You can also return NAs for these genes (`when_null_selected = "NA"`).

The default option is `when_null_selected = "ones"`. This simply calls all these genes non-significant, which in most cases is sufficient since we are more interested in genes with strong evidence for differential exon usage. However, if it is important to rank the low-evidence genes in your data set, use the `when_null_selected = "full"` option. If `when_null_selected = "ones"` or `when_null_selected = "NA"`, the "full" fitted models are not required.

The results object contains gene names, fitted model results, raw p-values, multiple testing adjusted p-values (Benjamini-Hochberg false discovery rates, FDR), likelihood ratio (LR) test statistics, and degrees of freedom of the LR tests.

For more details, see `?LRTests`.

\vspace{6pt}

```{r}
rs_results <- LRTests(rs_results)
```



### Summary table of results

The function `summaryTable()` is used to generate a summary table of the results.

The results are displayed as a data frame of the top `n` most highly significant genes, ranked according to either the false discovery rate (FDR) or raw p-values, up to a specified significance threshold (e.g. FDR < 0.05).

The argument `rank_by` chooses whether to rank by FDR or raw p-values.

To display results for all genes up to the significance threshold, set the argument `n = Inf`. To display results for all genes in the data set, set both `n = Inf` and `threshold = 1`.

For more details, see `?summaryTable`.

\vspace{6pt}

```{r}
summaryTable(rs_results)
```




# Analyze results

For the simulated data set in this vignette, the true differential splicing status of each gene is known. In this section, we show how to analyze the results and calculate a contingency table showing the number of true positives, true negatives, false positives, and false negatives.


## Summary of all significant genes

As shown in the workflow above, we can use the `summaryTable()` function with argument `n = Inf` to display a list of all genes with significant evidence for differential exon usage (DEU).

\vspace{6pt}

```{r}
summaryTable(rs_results, n = Inf)
```


The total number of genes with significant evidence for DEU at a given threshold can also be calculated.

Note that we are using the multiple testing adjusted p-values (Benjamini-Hochberg false discovery rates, FDRs) for this calculation. A standard threshold of FDR < 0.05 implies that 5% of genes in the list are expected to be false discoveries.

\vspace{6pt}

```{r}
sum(p_adj(rs_results) < 0.05)

table(p_adj(rs_results) < 0.05)
```



## Contingency table

As mentioned above, the true differential splicing (DS) status is known for each gene, since this is a simulated data set. Therefore, we can calculate contingency tables comparing the true and predicted DS status for each gene at a given significance threshold. Increasing the significance threshold returns more genes, at the expense of a larger number of false positives.

\vspace{6pt}

```{r}
# load true DS status labels
file_truth <- system.file("extdata/vignette_truth.txt", package = "regsplice")
data_truth <- read.table(file_truth, header = TRUE, sep = "\t", stringsAsFactors = FALSE)

str(data_truth)

# remove genes that were filtered during regsplice analysis
data_truth <- data_truth[data_truth$gene %in% gene_IDs(rs_results), ]

dim(data_truth)

length(gene_IDs(rs_results))

# number of true DS genes in simulated data set
sum(data_truth$ds_status == 1)

table(data_truth$ds_status)

# contingency table comparing true and predicted DS status for each gene
# (significance threshold: FDR < 0.05)
table(true = data_truth$ds_status, predicted = p_adj(rs_results) < 0.05)

# increasing the threshold detects more genes, at the expense of more false positives
table(true = data_truth$ds_status, predicted = p_adj(rs_results) < 0.99)
```




# Additional information


## Additional user options

Additional user options not discussed in the workflow above include:

- `alpha`: Elastic net parameter for `glmnet` model fitting functions. The value of `alpha` must be between 0 (ridge regression) and 1 (lasso). The default value is 1, which fits a lasso model. See `glmnet` package documentation for more details.

<br/>

- `lambda_choice`: Parameter to select which optimal `lambda` value to choose from the `cv.glmnet` cross validation fit. Available choices are `"lambda.min"` (model with minimum cross-validated error) and `"lambda.1se"` (most regularized model with cross-validated error within one standard error of minimum). The default value is `"lambda.min"`. See `glmnet` package documentation for more details.

<br/>

For further details, including a complete list and description of all available user options, refer to the documentation for the `regsplice()` wrapper function, which can be accessed with `?regsplice` or `help(regsplice)`.



## Design matrices

The function `createDesignMatrix()` creates the model design matrix for each gene. This function is called automatically by the model fitting functions, so does not need to be used directly. In this section, we demonstrate how it works for a single gene, and show an example design matrix, in order to provide further insight into the statistical methodology.

The design matrix includes main effect terms for each exon and each sample, and interaction terms between the exons and conditions.

Note that the design matrix does not include main effect terms for the conditions, since these are absorbed into the main effect terms for the samples. In addition, the design matrix does not include an intercept column, since it is simpler to let the model fitting functions add an intercept term later.

For more details, see `?createDesignMatrix`.

\vspace{6pt}

```{r}
# gene with 3 exons
# 4 biological samples; 2 samples in each of 2 conditions
design_example <- createDesignMatrix(condition = rep(c(0, 1), each = 2), n_exons = 3)

design_example
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/initializeResults.R
\name{initializeResults}
\alias{initializeResults}
\title{Initialize RegspliceResults object.}
\usage{
initializeResults(rs_data)
}
\arguments{
\item{rs_data}{\code{\linkS4class{RegspliceData}} object. This should contain gene IDs
in a column named \code{gene_IDs} in the row meta-data, which can be accessed with 
the accessor function \code{\link{rowData}}.}
}
\value{
Returns a \code{\linkS4class{RegspliceResults}} object containing gene IDs 
  only.
}
\description{
Initialize a \code{RegspliceResults} object, which will contain the results of the 
\code{regsplice} analysis.
}
\details{
Creates a \code{\linkS4class{RegspliceResults}} object containing gene names only. 
This object will subsequently be populated using the functions 
\code{\link{fitRegMultiple}}, \code{\link{fitNullMultiple}}, 
\code{\link{fitFullMultiple}}, and \code{\link{LRTests}}.

Previous step: Calculate \code{limma-voom} transformation and weights with 
\code{\link{runVoom}}.
Next step: Fit models with \code{\link{fitRegMultiple}}, 
\code{\link{fitNullMultiple}}, and \code{\link{fitFullMultiple}}.
}
\examples{
file_counts <- system.file("extdata/vignette_counts.txt", package = "regsplice")
data <- read.table(file_counts, header = TRUE, sep = "\t", stringsAsFactors = FALSE)
head(data)

counts <- data[, 2:7]
tbl_exons <- table(sapply(strsplit(data$exon, ":"), function(s) s[[1]]))
gene_IDs <- names(tbl_exons)
n_exons <- unname(tbl_exons)
condition <- rep(c("untreated", "treated"), each = 3)

rs_data <- RegspliceData(counts, gene_IDs, n_exons, condition)

rs_data <- filterZeros(rs_data)
rs_data <- filterLowCounts(rs_data)
rs_data <- runNormalization(rs_data)
rs_data <- runVoom(rs_data)

rs_results <- initializeResults(rs_data)

}
\seealso{
\code{\linkS4class{RegspliceData}} \code{\linkS4class{RegspliceResults}} 
  \code{\link{fitRegMultiple}} \code{\link{fitNullMultiple}} 
  \code{\link{fitFullMultiple}} \code{\link{LRTests}} \code{\link{summaryTable}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/class_RegspliceData.R
\docType{class}
\name{RegspliceData-class}
\alias{RegspliceData-class}
\alias{RegspliceData}
\alias{assays,RegspliceData-method}
\alias{countsData}
\alias{countsData,RegspliceData-method}
\alias{weightsData}
\alias{weightsData,RegspliceData-method}
\alias{rowData,RegspliceData-method}
\alias{colData,RegspliceData-method}
\alias{[,RegspliceData,ANY,ANY,ANY-method}
\title{RegspliceData objects.}
\usage{
RegspliceData(counts, gene_IDs = NULL, n_exons = NULL, condition = NULL)

\S4method{assays}{RegspliceData}(x, withDimnames, ..., value)

countsData(x)

\S4method{countsData}{RegspliceData}(x)

weightsData(x)

\S4method{weightsData}{RegspliceData}(x)

\S4method{rowData}{RegspliceData}(x)

\S4method{colData}{RegspliceData}(x, ..., value)

\S4method{[}{RegspliceData,ANY,ANY,ANY}(x, i, j)
}
\arguments{
\item{counts}{RNA-seq read counts or exon microarray intensities (matrix or data 
frame). Rows are exons, and columns are biological samples. Alternatively,
\code{counts} also accepts a \code{SummarizedExperiment} input object containing all
required input data, which may be useful when running \code{regsplice} as part of a
pipeline with other packages.}

\item{gene_IDs}{Vector of gene IDs (character vector). Length is equal to the number 
of genes.}

\item{n_exons}{Vector of exon lengths (numeric vector of integers), i.e. the number of
exon bins per gene. Length is equal to the number of genes.}

\item{condition}{Experimental condition for each biological sample (character or 
numeric vector, or factor).}

\item{x}{\code{RegspliceData} object (for accessor or subsetting functions).}

\item{withDimnames}{See \code{SummarizedExperiment::assays()}.}

\item{...}{Additional arguments for replacement with \code{`[<-`}.}

\item{value}{Value for replacement with \code{`[<-`}.}

\item{i}{Gene names (character vector) or row numbers (numeric vector) for subsetting 
genes or exons. Note that when subsetting whole genes, gene names (character vector)
should be provided instead of row numbers, to avoid possible errors due to selecting
incorrect row numbers. Row numbers may be provided to subset individual exons.}

\item{j}{Column numbers (numeric vector) for subsetting biological samples.}
}
\value{
Returns a \code{RegspliceData} object.
}
\description{
\code{RegspliceData} objects contain data in the format required by functions in the 
\code{regsplice} analysis pipeline.
}
\details{
The \code{RegspliceData} format is based on the 
\code{\link[SummarizedExperiment]{SummarizedExperiment}} container. Initially, objects
contain raw data along with meta-data for rows (genes and exons) and columns 
(biological samples). During subsequent steps in the \code{regsplice} analysis 
pipeline, the data values are modified, and additional data and meta-data are added to
the object. Final results are stored in a \code{\linkS4class{RegspliceResults}} 
object.

\code{RegspliceData} objects are created with the constructor function 
\code{RegspliceData()}.

Required inputs for the constructor function are \code{counts} (matrix or data frame
of RNA-seq read counts or exon microarray intensities), \code{gene_IDs} (vector of
gene IDs), \code{n_exons} (vector of exon lengths, i.e. number of exon bins per gene),
and \code{condition} (vector of experimental conditions for each biological sample).

Alternatively, the inputs can be provided as a \code{SummarizedExperiment} object, 
which will be parsed to extract each of these components. This may be useful when 
running \code{regsplice} as part of a pipeline together with other packages.

See the vignette for an example showing how to construct \code{gene_IDs} and 
\code{n_exons} from a column of gene:exon IDs.

Exon microarray intensities should be log2-transformed, which is usually done during 
pre-processing of microarray data. (RNA-seq counts will be transformed automatically
during the \code{regsplice} analysis pipeline; see \code{\link{runVoom}}.)

After creating a \code{RegspliceData} object, the wrapper function 
\code{\link{regsplice}} can be used to run the analysis pipeline with a single
command. Alternatively, you can run the individual functions for each step in the
pipeline, beginning with \code{\link{filterZeros}} (see vignette for details).
}
\section{Fields}{

\describe{
\item{\code{counts}}{Matrix of RNA-seq read counts or exon microarray intensities. Rows are 
exons, and columns are biological samples.}

\item{\code{weights}}{(Optional) Matrix of observation-level weights. Rows are exons, and 
columns are biological samples. Created by the \code{\link{runVoom}} function.}

\item{\code{rowData}}{\code{DataFrame} of row meta-data. This should contain two columns: 
\code{gene_IDs} and \code{exon_IDs}, which are created by the \code{RegspliceData}
constructor function.}

\item{\code{colData}}{\code{DataFrame} of column meta-data. This contains the experimental 
condition and (optionally) normalization factors for each biological sample. 
Normalization factors are created by the \code{\link{runVoom}} function.}
}}

\section{Accessor functions}{


\itemize{
\item \code{countsData()}: Accesses the \code{counts} data matrix.
\item \code{weightsData()}: Accesses the (optional) \code{weights} data matrix.
\item \code{rowData()}: Accesses the \code{DataFrame} of row meta-data. This should
contain two columns: \code{gene_IDs} and \code{exon_IDs}.
\item \code{colData()}: Accesses the \code{DataFrame} of column meta-data. This
contains the experimental condition and (optionally) normalization factors for each
biological sample.
}
}

\section{Subsetting}{


Subsetting of \code{RegspliceData} objects is performed with square brackets, 
\code{x[i, j]}, where \code{x} is the name of the object. The subsetting operations
are designed to keep data and meta-data in sync.

For subsetting by rows, there are two possibilities:
\itemize{
\item Subsetting genes: To subset whole genes, provide a character vector of gene 
names to the argument \code{i}. The returned object will contain all rows 
corresponding to these genes. Row numbers should not be used when subsetting whole
genes, since this risks potential errors due to selecting incorrect rows.
\item Subsetting exons: To subset individual exons, provide the corresponding row
numbers to the argument \code{i}.
}

For subsetting by columns (biological samples), provide the corresponding column
numbers to the argument \code{j}.
}

\examples{
# ---------
# Example 1
# ---------

counts <- matrix(sample(100:200, 14 * 6, replace = TRUE), nrow = 14, ncol = 6)
gene_IDs <- paste0("gene", 1:5)
n_exons <- c(3, 2, 3, 1, 5)
condition <- rep(c(0, 1), each = 3)

rs_data <- RegspliceData(counts, gene_IDs, n_exons, condition)

rs_data
countsData(rs_data)
rowData(rs_data)
colData(rs_data)

rs_data[1, ]
rs_data[1, 1:3]

rs_data["gene1", ]
rs_data["gene1", 1:3]


# --------------------
# Example 2 (Vignette)
# --------------------

file_counts <- system.file("extdata/vignette_counts.txt", package = "regsplice")
data <- read.table(file_counts, header = TRUE, sep = "\t", stringsAsFactors = FALSE)
head(data)

counts <- data[, 2:7]
tbl_exons <- table(sapply(strsplit(data$exon, ":"), function(s) s[[1]]))
gene_IDs <- names(tbl_exons)
n_exons <- unname(tbl_exons)
condition <- rep(c("untreated", "treated"), each = 3)

rs_data <- RegspliceData(counts, gene_IDs, n_exons, condition)

rs_data
head(countsData(rs_data))
rowData(rs_data)
colData(rs_data)

rs_data[1, ]
rs_data[1, 1:3]

rs_data["ENSG00000000003", ]
rs_data["ENSG00000000003", 1:3]

}
\seealso{
\code{\link{regsplice}} \code{\link{filterZeros}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fitting_functions_multiple_genes.R
\name{fitRegMultiple}
\alias{fitRegMultiple}
\alias{fitNullMultiple}
\alias{fitFullMultiple}
\title{Fit models.}
\usage{
fitRegMultiple(
  rs_results,
  rs_data,
  alpha = 1,
  lambda_choice = c("lambda.min", "lambda.1se"),
  seed = NULL,
  ...
)

fitNullMultiple(rs_results, rs_data, seed = NULL, ...)

fitFullMultiple(rs_results, rs_data, seed = NULL, ...)
}
\arguments{
\item{rs_results}{\code{\linkS4class{RegspliceResults}} object, which will be used to 
store results. Initialized using the constructor function \code{RegspliceResults()}.
See \code{\linkS4class{RegspliceResults}} for details.}

\item{rs_data}{\code{\linkS4class{RegspliceData}} object. In the case of RNA-seq read 
count data, this has been pre-transformed with \code{\link{runVoom}}. Contains 
\code{counts} and \code{weights} data matrices, and vector of experimental 
conditions for each biological sample stored in \code{colData}. See 
\code{\linkS4class{RegspliceData}} for details.}

\item{alpha}{Elastic net parameter \code{alpha} for \code{glmnet} model fitting 
functions. Must be between 0 (ridge regression) and 1 (lasso). Default is 1 (lasso).
See \code{glmnet} documentation for more details.}

\item{lambda_choice}{Parameter to select which optimal \code{lambda} value to choose 
from the \code{cv.glmnet} cross validation fit. Choices are "lambda.min" (model with
minimum cross-validated error) and "lambda.1se" (most regularized model with 
cross-validated error within one standard error of minimum). Default is 
"lambda.min". See \code{glmnet} documentation for more details.}

\item{seed}{Random seed (integer). Default is NULL. Provide an integer value to set 
the random seed for reproducible results.}

\item{...}{Other arguments to pass to \code{cv.glmnet}, \code{glmnet}, or \code{glm}.}
}
\value{
Returns a \code{\linkS4class{RegspliceResults}} object containing deviance and
  degrees of freedom of the fitted models. See \code{\linkS4class{RegspliceResults}}
  for details.
}
\description{
Model fitting functions for \code{regsplice} package.
}
\details{
There are three model fitting functions:

\code{fitRegMultiple} fits regularized (lasso) models containing an optimal subset of 
exon:condition interaction terms for each gene. The model fitting procedure penalizes 
the interaction terms only, so that the main effect terms for exons and samples are 
always included. This ensures that the null model is nested, allowing likelihood ratio
tests to be calculated.

\code{fitNullMultiple} fits the null models, which do not contain any interaction 
terms.

\code{fitFullMultiple} fits full models, which contain all exon:condition interaction
terms for each gene.

See \code{\link{createDesignMatrix}} for more details about the terms in each model.

The fitting functions fit models for all genes in the data set.

A random seed can be provided with the \code{seed} argument, to generate reproducible
results.

If the \code{rs_data} object does not contain a weights matrix, all exon bins are 
weighted equally.

Previous step: Initialize \code{\linkS4class{RegspliceResults}} object with 
\code{\link{initializeResults}}.
Next step: Calculate likelihood ratio tests with \code{\link{LRTests}}.
}
\examples{
file_counts <- system.file("extdata/vignette_counts.txt", package = "regsplice")
data <- read.table(file_counts, header = TRUE, sep = "\t", stringsAsFactors = FALSE)
head(data)

counts <- data[, 2:7]
tbl_exons <- table(sapply(strsplit(data$exon, ":"), function(s) s[[1]]))
gene_IDs <- names(tbl_exons)
n_exons <- unname(tbl_exons)
condition <- rep(c("untreated", "treated"), each = 3)

rs_data <- RegspliceData(counts, gene_IDs, n_exons, condition)

rs_data <- filterZeros(rs_data)
rs_data <- filterLowCounts(rs_data)
rs_data <- runNormalization(rs_data)
rs_data <- runVoom(rs_data)

rs_results <- initializeResults(rs_data)

rs_results <- fitRegMultiple(rs_results, rs_data)
rs_results <- fitNullMultiple(rs_results, rs_data)
rs_results <- fitFullMultiple(rs_results, rs_data)

}
\seealso{
\code{\link{createDesignMatrix}} \code{\link{RegspliceResults}} 
  \code{\link{initializeResults}} \code{\link{LRTests}}

\code{\link[glmnet]{glmnet}} \code{\link[glmnet]{cv.glmnet}} 
  \code{\link[stats]{glm}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summaryTable.R
\name{summaryTable}
\alias{summaryTable}
\title{Summary table.}
\usage{
summaryTable(
  rs_results,
  n = 20,
  threshold = 0.05,
  rank_by = c("FDR", "p-value", "none")
)
}
\arguments{
\item{rs_results}{\code{\linkS4class{RegspliceResults}} object containing results of a
\code{regsplice} analysis, generated with wrapper function \code{\link{regsplice}} 
(or individual functions up to \code{\link{LRTests}}). See 
\code{\linkS4class{RegspliceResults}} for details.}

\item{n}{Number of genes to display in summary table. Default is 20. If the total
number of significant genes up to the significance threshold is less than \code{n},
only the significant genes are shown. Set to \code{Inf} to display all significant
genes; or set both \code{n = Inf} and \code{threshold = 1} to display all genes in
the data set.}

\item{threshold}{Significance threshold (for either FDR or raw p-values, depending on 
choice of argument \code{rank_by}). Default is 0.05. Set to 1 to display all 
\code{n} genes; or set both \code{n = Inf} and \code{threshold = 1} to display all 
genes in the data set.}

\item{rank_by}{Whether to rank genes by false discovery rate (FDR), raw p-values, or 
no ranking. Choices are \code{"FDR"}, \code{"p-value"}, and \code{"none"}. Default 
is \code{"FDR"}.}
}
\value{
Returns a data frame containing results for the top \code{n} most highly 
  significant genes, up to the specified significance threshold for the FDR or raw 
  p-values.
}
\description{
Display summary table of results from a \code{regsplice} analysis.
}
\details{
The results of a \code{regsplice} analysis consist of a set of multiple testing 
adjusted p-values (Benjamini-Hochberg false discovery rates, FDR) quantifying the 
statistical evidence for differential exon usage (DEU) for each gene. Typically, the
adjusted p-values are used to rank the genes in the data set according to their
evidence for DEU, and an appropriate significance threshold (e.g. FDR < 0.05) can be
used to generate a list of genes with statistically significant evidence for DEU.

The main \code{regsplice} functions return results in the form of a 
\code{\linkS4class{RegspliceResults}} object, which contains slots for gene names, 
fitted model results, raw p-values, multiple testing adjusted p-values 
(Benjamini-Hochberg FDR), likelihood ratio (LR) test statistics, and degrees of 
freedom of the LR tests. See \code{\linkS4class{RegspliceResults}} and the main 
\code{regsplice} wrapper function \code{\link{regsplice}} for details.

This function generates a summary table of the results. The results are displayed as a
data frame of the top \code{n} most highly significant genes, ranked according to 
either FDR or raw p-values, up to a specified significance threshold (e.g. FDR < 
0.05).

The argument \code{rank_by} controls whether to rank by FDR or raw p-values. The
default is to rank by FDR.

To display results for all genes up to the significance threshold, set the argument 
\code{n = Inf}. To display results for all genes in the data set, set both \code{n = 
Inf} and \code{threshold = 1}.

Previous step: Run \code{regsplice} pipeline with the \code{\link{regsplice}} wrapper
function (or individual functions up to \code{\link{LRTests}}).
}
\examples{
file_counts <- system.file("extdata/vignette_counts.txt", package = "regsplice")
data <- read.table(file_counts, header = TRUE, sep = "\t", stringsAsFactors = FALSE)
head(data)

counts <- data[, 2:7]
tbl_exons <- table(sapply(strsplit(data$exon, ":"), function(s) s[[1]]))
gene_IDs <- names(tbl_exons)
n_exons <- unname(tbl_exons)
condition <- rep(c("untreated", "treated"), each = 3)

rs_data <- RegspliceData(counts, gene_IDs, n_exons, condition)

rs_results <- regsplice(rs_data)

summaryTable(rs_results)
summaryTable(rs_results, n = Inf, threshold = 1)

}
\seealso{
\code{\linkS4class{RegspliceResults}} \code{\link{regsplice}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/filterZeros.R
\name{filterZeros}
\alias{filterZeros}
\title{Filter zero-count exons.}
\usage{
filterZeros(rs_data)
}
\arguments{
\item{rs_data}{\code{\linkS4class{RegspliceData}} object.}
}
\value{
Returns a \code{\linkS4class{RegspliceData}} object.
}
\description{
Filter exons with zero RNA-seq read counts in all biological samples.
}
\details{
Removes exon bins with zero RNA-seq read counts in all biological samples. Any 
remaining single-exon genes (after filtering) are also removed (since differential
splicing requires multiple exon bins).

Input data is assumed to be in the form of a \code{RegspliceData} object. See 
\code{\link{RegspliceData}} for details.

After filtering zero-count exon bins, any remaining genes containing only a single
exon bin are also removed (since differential splicing requires multiple exon bins).

Filtering should be skipped when using exon microarray data. (When using the 
\code{regsplice} wrapper function, filtering can be disabled with the argument 
\code{filter = FALSE}).

Previous step: Create \code{RegspliceData} object with \code{\link{RegspliceData}}
constructor function.
Next step: Filter low-count exon bins with \code{\link{filterLowCounts}}.
}
\examples{
file_counts <- system.file("extdata/vignette_counts.txt", package = "regsplice")
data <- read.table(file_counts, header = TRUE, sep = "\t", stringsAsFactors = FALSE)
head(data)

counts <- data[, 2:7]
tbl_exons <- table(sapply(strsplit(data$exon, ":"), function(s) s[[1]]))
gene_IDs <- names(tbl_exons)
n_exons <- unname(tbl_exons)
condition <- rep(c("untreated", "treated"), each = 3)

rs_data <- RegspliceData(counts, gene_IDs, n_exons, condition)

rs_data <- filterZeros(rs_data)

}
\seealso{
\code{\linkS4class{RegspliceData}} \code{\link{filterLowCounts}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/createDesignMatrix.R
\name{createDesignMatrix}
\alias{createDesignMatrix}
\title{Create design matrix.}
\usage{
createDesignMatrix(condition, n_exons)
}
\arguments{
\item{condition}{Experimental conditions for each sample (character or numeric vector,
or factor).}

\item{n_exons}{Number of exons in the gene (integer).}
}
\value{
Returns a model design matrix for the gene, in the format required by the 
  \code{regsplice} model fitting functions.
}
\description{
Create a model design matrix for a single gene.
}
\details{
Creates a model design matrix for a single gene in the format required by the 
\code{regsplice} model fitting functions. Required inputs are the experimental 
conditions (groups) for each sample, and the number of exons in the gene.

The design matrix includes main effect terms for each exon and each sample, and
interaction terms between the exons and conditions.

Note that the design matrix does not include main effect terms for the conditions,
since these are absorbed into the main effect terms for the samples. In addition, the
design matrix does not include an intercept column, since it is simpler to let the
model fitting functions add an intercept term later.

The model fitting functions in subsequent steps call this function once for each gene.
}
\examples{
condition <- rep(c(0, 1), each = 3)
n_exons <- 10
X <- createDesignMatrix(condition, n_exons)

}
\seealso{
\code{\link{fitRegMultiple}} \code{\link{fitNullMultiple}} 
  \code{\link{fitFullMultiple}} \code{\link{LRTests}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/runVoom.R
\name{runVoom}
\alias{runVoom}
\title{Calculate 'voom' transformation and weights.}
\usage{
runVoom(rs_data)
}
\arguments{
\item{rs_data}{\code{\linkS4class{RegspliceData}} object, which has been filtered with
\code{\link{filterZeros}} and \code{\link{filterLowCounts}}, and (optionally) 
normalization factors added with \code{\link{runNormalization}}.}
}
\value{
Returns a \code{\linkS4class{RegspliceData}} object. Transformed counts are 
  stored in the \code{counts} matrix, and weights are stored in a new \code{weights} 
  data matrix. The data matrices can be accessed with the accessor functions 
  \code{\link{countsData}} and \code{\link{weightsData}}.
}
\description{
Use \code{limma-voom} to transform counts and calculate exon-level weights.
}
\details{
Raw counts do not fulfill the statistical assumptions required for linear modeling. 
The \code{limma-voom} methodology transforms counts to log2-counts per million 
(logCPM), and calculates exon-level weights based on the observed mean-variance 
relationship. Linear modeling methods can then be applied.

For more details, see the documentation for \code{\link[limma]{voom}} in the 
\code{limma} package.

Note that \code{voom} assumes that exon bins (rows) with zero or low counts have 
already been removed, so this step should be done after filtering with 
\code{\link{filterZeros}} and \code{\link{filterLowCounts}}.

Normalization factors can be provided in a column named \code{norm_factors} in the 
column meta-data (\code{colData} slot) of the \code{\linkS4class{RegspliceData}} 
object. These will be used by \code{voom} to calculate normalized library sizes. If 
normalization factors are not provided, \code{voom} will use non-normalized library 
sizes (columnwise total counts) instead.

The experimental conditions or group labels for each biological sample are assumed to 
be in a column named \code{condition} in the column meta-data (\code{colData} slot) of
the \code{\linkS4class{RegspliceData}} object. This column is created when the object 
is initialized with the \code{RegspliceData()} constructor function.

The transformed counts are stored in the updated \code{counts} matrix, which can be 
accessed with the \code{\link{countsData}} accessor function. The weights are stored 
in a new data matrix labeled \code{weights}, which can be accessed with the 
\code{\link{weightsData}} accessor function. In addition, the normalized library sizes
(if available) are stored in a new column named \code{lib_sizes} in the column 
meta-data (\code{colData} slot).

If you are using exon microarray data, this step should be skipped, since exon 
microarray intensities are already on a continuous scale.

Previous step: Calculate normalization factors with \code{\link{runNormalization}}.
Next step: Initialize \code{\linkS4class{RegspliceResults}} object with the
constructor function \code{RegspliceResults()}.
}
\examples{
file_counts <- system.file("extdata/vignette_counts.txt", package = "regsplice")
data <- read.table(file_counts, header = TRUE, sep = "\t", stringsAsFactors = FALSE)
head(data)

counts <- data[, 2:7]
tbl_exons <- table(sapply(strsplit(data$exon, ":"), function(s) s[[1]]))
gene_IDs <- names(tbl_exons)
n_exons <- unname(tbl_exons)
condition <- rep(c("untreated", "treated"), each = 3)

rs_data <- RegspliceData(counts, gene_IDs, n_exons, condition)

rs_data <- filterZeros(rs_data)
rs_data <- filterLowCounts(rs_data)
rs_data <- runNormalization(rs_data)
rs_data <- runVoom(rs_data)

}
\seealso{
\code{\link{runNormalization}} \code{\link{fitRegMultiple}} 
  \code{\link{fitNullMultiple}} \code{\link{fitFullMultiple}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/runNormalization.R
\name{runNormalization}
\alias{runNormalization}
\title{Calculate normalization factors.}
\usage{
runNormalization(rs_data, norm_method = "TMM")
}
\arguments{
\item{rs_data}{\code{\linkS4class{RegspliceData}} object, which has already been 
filtered with \code{\link{filterZeros}} and \code{\link{filterLowCounts}}.}

\item{norm_method}{Normalization method to use. Options are \code{"TMM"}, 
\code{"RLE"}, \code{"upperquartile"}, and \code{"none"}. See documentation for 
\code{\link[edgeR]{calcNormFactors}} in \code{edgeR} package for details. Default is
\code{"TMM"}.}
}
\value{
Returns a \code{\linkS4class{RegspliceData}} object. Normalization factors are
  stored in the column \code{norm_factors} in the column meta-data (\code{colData} 
  slot), which can be accessed with the \code{colData()} accessor function.
}
\description{
Calculate normalization factors to scale library sizes, using the TMM (trimmed mean of
M-values) method implemented in \code{edgeR}.
}
\details{
Normalization factors are used to scale the raw library sizes (total read counts per 
sample). We use the TMM (trimmed mean of M-values) normalization method (Robinson and 
Oshlack, 2010), as implemented in the \code{edgeR} package.

For more details, see the documentation for \code{\link[edgeR]{calcNormFactors}} in 
the \code{edgeR} package.

This step should be performed after filtering with \code{\link{filterZeros}} and 
\code{\link{filterLowCounts}}. The normalization factors are then used by 
\code{limma-voom} in the next step (\code{\link{runVoom}}).

The normalization factors are stored in a new column named \code{norm_factors} in the 
column meta-data (\code{colData} slot) of the \code{\linkS4class{RegspliceData}} 
object. The \code{colData} can be accessed with the accessor function 
\code{colData()}.

Normalization should be skipped when using exon microarray data. (When using the 
\code{\link{regsplice}} wrapper function, normalization can be disabled with the 
argument \code{normalize = FALSE}).

Previous step: Filter low-count exon bins with \code{\link{filterLowCounts}}.
Next step: Calculate \code{limma-voom} transformation and weights with 
\code{\link{runVoom}}.
}
\examples{
file_counts <- system.file("extdata/vignette_counts.txt", package = "regsplice")
data <- read.table(file_counts, header = TRUE, sep = "\t", stringsAsFactors = FALSE)
head(data)

counts <- data[, 2:7]
tbl_exons <- table(sapply(strsplit(data$exon, ":"), function(s) s[[1]]))
gene_IDs <- names(tbl_exons)
n_exons <- unname(tbl_exons)
condition <- rep(c("untreated", "treated"), each = 3)

rs_data <- RegspliceData(counts, gene_IDs, n_exons, condition)

rs_data <- filterZeros(rs_data)
rs_data <- filterLowCounts(rs_data)
rs_data <- runNormalization(rs_data)

}
\seealso{
\code{\link{filterLowCounts}} \code{\link{runVoom}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/filterLowCounts.R
\name{filterLowCounts}
\alias{filterLowCounts}
\title{Filter low-count exons.}
\usage{
filterLowCounts(rs_data, filter_min_per_exon = 6, filter_min_per_sample = 3)
}
\arguments{
\item{rs_data}{\code{\linkS4class{RegspliceData}} object.}

\item{filter_min_per_exon}{Filtering parameter: minimum number of reads per exon bin, 
summed across all biological samples. Default is 6.}

\item{filter_min_per_sample}{Filtering parameter: minimum number of reads per 
biological sample; i.e. for each exon bin, at least one sample must have this number
of reads. Default is 3.}
}
\value{
Returns a \code{\linkS4class{RegspliceData}} object.
}
\description{
Filter low-count exons from RNA-seq read count data.
}
\details{
Filters low-count exon bins from RNA-seq read count data. Any remaining single-exon 
genes (after filtering) are also removed (since differential splicing requires
multiple exon bins).

Input data is assumed to be in the form of a \code{RegspliceData} object. See 
\code{\link{RegspliceData}} for details.

The arguments \code{filter_min_per_exon} and \code{filter_min_per_sample} control the 
amount of filtering. Exon bins that meet the filtering conditions are kept. Default 
values for the arguments are provided; however, these should be adjusted depending on 
the total number of samples and the number of samples per condition.

After filtering low-count exon bins, any remaining genes containing only a single exon
bin are also removed (since differential splicing requires multiple exon bins).

Filtering should be skipped when using exon microarray data. (When using the 
\code{regsplice} wrapper function, filtering can be disabled with the argument 
\code{filter = FALSE}).

Previous step: Filter zero-count exon bins with \code{\link{filterZeros}}.
Next step: Calculate normalization factors with \code{\link{runNormalization}}.
}
\examples{
file_counts <- system.file("extdata/vignette_counts.txt", package = "regsplice")
data <- read.table(file_counts, header = TRUE, sep = "\t", stringsAsFactors = FALSE)
head(data)

counts <- data[, 2:7]
tbl_exons <- table(sapply(strsplit(data$exon, ":"), function(s) s[[1]]))
gene_IDs <- names(tbl_exons)
n_exons <- unname(tbl_exons)
condition <- rep(c("untreated", "treated"), each = 3)

rs_data <- RegspliceData(counts, gene_IDs, n_exons, condition)

rs_data <- filterZeros(rs_data)
rs_data <- filterLowCounts(rs_data)

}
\seealso{
\code{\link{filterZeros}} \code{\link{runNormalization}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/regsplice_wrapper.R
\name{regsplice}
\alias{regsplice}
\title{Wrapper function to run regsplice.}
\usage{
regsplice(
  rs_data,
  filter_zeros = TRUE,
  filter_low_counts = TRUE,
  filter_min_per_exon = 6,
  filter_min_per_sample = 3,
  normalize = TRUE,
  norm_method = "TMM",
  voom = TRUE,
  alpha = 1,
  lambda_choice = c("lambda.min", "lambda.1se"),
  when_null_selected = c("ones", "full", "NA"),
  seed = NULL,
  ...
)
}
\arguments{
\item{rs_data}{\code{RegspliceData} object containing input data. See 
\code{\linkS4class{RegspliceData}} for details.}

\item{filter_zeros}{Whether to filter zero-count exon bins, using 
\code{\link{filterZeros}}. Default is TRUE. Set to FALSE for exon microarray data.}

\item{filter_low_counts}{Whether to filter low-count exon bins, using 
\code{\link{filterLowCounts}}. Default is TRUE. Set to FALSE for exon microarray 
data.}

\item{filter_min_per_exon}{Filtering parameter for low-count exon bins: minimum number
of reads per exon bin, summed across all biological samples. Default is 6. See 
\code{\link{filterLowCounts}} for details.}

\item{filter_min_per_sample}{Filtering parameter for low-count exon bins: minimum 
number of reads per biological sample; i.e. for each exon bin, at least one sample 
must have this number of reads. Default is 3. See \code{\link{filterLowCounts}} for
details.}

\item{normalize}{Whether to calculate normalization factors, using 
\code{\link{runNormalization}}. Default is TRUE. If FALSE, non-normalized library 
sizes will be used. Set to FALSE for exon microarray data.}

\item{norm_method}{Normalization method to use. Options are \code{"TMM"}, 
\code{"RLE"}, \code{"upperquartile"}, and \code{"none"}. Default is \code{"TMM"}. 
See \code{\link{runNormalization}} for details.}

\item{voom}{Whether to calculate \code{limma-voom} transformation and weights, using 
\code{\link{runVoom}}. Default is TRUE. If FALSE, model fitting functions will use 
the raw input data (not recommended for count data) with exon bins weighted equally.
Set to FALSE for exon microarray data.}

\item{alpha}{Elastic net parameter \code{alpha} for \code{glmnet} model fitting 
functions. Must be between 0 (ridge regression) and 1 (lasso). Default is 1 (lasso).
See \code{glmnet} documentation for more details.}

\item{lambda_choice}{Parameter to select which optimal \code{lambda} value to choose 
from the \code{cv.glmnet} cross validation fit. Choices are "lambda.min" (model with
minimum cross-validated error) and "lambda.1se" (most regularized model with 
cross-validated error within one standard error of minimum). Default is 
"lambda.min". See \code{glmnet} documentation for more details.}

\item{when_null_selected}{Which option to use for genes where the lasso model selects 
zero interaction terms, i.e. identical to the null model. Options are \code{"ones"},
\code{"full"}, and \code{"NA"}. Default is \code{"ones"}. See \code{\link{LRTests}} 
for details.}

\item{seed}{Random seed (integer). Default is NULL. Provide an integer value to set 
the random seed for reproducible results.}

\item{...}{Other arguments to pass to \code{cv.glmnet}, \code{glmnet}, or \code{glm}.}
}
\value{
Returns a \code{\linkS4class{RegspliceResults}} object containing fitted model
  results and likelihood ratio (LR) test results. The LR test results consist of the 
  following entries for each gene:
\itemize{
\item p_vals: raw p-values
\item p_adj: multiple testing adjusted p-values (Benjamini-Hochberg false discovery 
rates, FDR)
\item LR_stats: likelihood ratio test statistics
\item df_tests: degrees of freedom of likelihood ratio tests
}
}
\description{
Wrapper function to run a \code{regsplice} analysis with a single command.
}
\details{
This wrapper function runs the \code{regsplice} analysis pipeline with a single 
command.

The required input format is a \code{RegspliceData} object, which is created with the 
\code{\linkS4class{RegspliceData}} constructor function.

The wrapper function calls each of the individual functions in the analysis pipeline 
in sequence. You can also run the individual functions directly, which provides 
additional flexibility and insight into the statistical methodology. See the vignette 
for a description of the individual functions and an example workflow.

After running the analysis pipeline, a summary table of the results can be displayed 
with \code{\link{summaryTable}}.

Note that when using exon microarray data, the filtering, normalization, and 
\code{voom} steps should be disabled with the respective arguments.

See \code{\linkS4class{RegspliceData}} for details on constructing the input data 
object; \code{\link{filterZeros}} and \code{\link{filterLowCounts}} for details about
filtering; \code{\link{runNormalization}} and \code{\link{runVoom}} for details about
calculation of normalization factors and \code{voom} transformation and weights;
\code{\link{createDesignMatrix}} for details about the model design matrices;
\code{\link{fitRegMultiple}}, \code{\link{fitNullMultiple}}, or 
\code{\link{fitFullMultiple}} for details about the model fitting functions; and 
\code{\link{LRTests}} for details about the likelihood ratio tests.
}
\examples{
file_counts <- system.file("extdata/vignette_counts.txt", package = "regsplice")
data <- read.table(file_counts, header = TRUE, sep = "\t", stringsAsFactors = FALSE)
head(data)

counts <- data[, 2:7]
tbl_exons <- table(sapply(strsplit(data$exon, ":"), function(s) s[[1]]))
gene_IDs <- names(tbl_exons)
n_exons <- unname(tbl_exons)
condition <- rep(c("untreated", "treated"), each = 3)

rs_data <- RegspliceData(counts, gene_IDs, n_exons, condition)

rs_results <- regsplice(rs_data)

summaryTable(rs_results)

}
\seealso{
\code{\linkS4class{RegspliceData}} \code{\linkS4class{RegspliceResults}} 
  \code{\link{initializeResults}} \code{\link{filterZeros}} 
  \code{\link{filterLowCounts}} \code{\link{runNormalization}} \code{\link{runVoom}}
  \code{\link{createDesignMatrix}} \code{\link{fitRegMultiple}}
  \code{\link{fitNullMultiple}} \code{\link{fitFullMultiple}} \code{\link{LRTests}}
  \code{\link{summaryTable}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/class_RegspliceResults.R
\docType{class}
\name{RegspliceResults-class}
\alias{RegspliceResults-class}
\alias{RegspliceResults}
\alias{gene_IDs}
\alias{gene_IDs,RegspliceResults-method}
\alias{p_vals}
\alias{p_vals,RegspliceResults-method}
\alias{p_adj}
\alias{p_adj,RegspliceResults-method}
\alias{LR_stats}
\alias{LR_stats,RegspliceResults-method}
\alias{df_tests}
\alias{df_tests,RegspliceResults-method}
\title{RegspliceResults objects.}
\usage{
RegspliceResults(gene_IDs)

gene_IDs(x)

\S4method{gene_IDs}{RegspliceResults}(x)

p_vals(x)

\S4method{p_vals}{RegspliceResults}(x)

p_adj(x)

\S4method{p_adj}{RegspliceResults}(x)

LR_stats(x)

\S4method{LR_stats}{RegspliceResults}(x)

df_tests(x)

\S4method{df_tests}{RegspliceResults}(x)
}
\arguments{
\item{gene_IDs}{Gene identifiers or names (character vector).}

\item{x}{\code{RegspliceResults} object (for accessor functions).}
}
\value{
Returns an empty \code{RegspliceResults} object.
}
\description{
\code{RegspliceResults} objects contain the results of a \code{regsplice} analysis.
}
\details{
\code{RegspliceResults} objects are created with the constructor function
\code{RegspliceResults()}, which requires the gene IDs as an argument.

Once created, \code{RegspliceResults} objects are then populated using the functions 
\code{\link{fitRegMultiple}}, \code{\link{fitNullMultiple}},
\code{\link{fitFullMultiple}}, and \code{\link{LRTests}}.

The function \code{\link{summaryTable}} can be used to display a summary table of the
results.
}
\section{Fields}{

\describe{
\item{\code{gene_IDs}}{Gene identifiers or names (character vector).}

\item{\code{fit_reg_dev}}{Deviance of fitted regularized (lasso) models from 
\code{\link{fitRegMultiple}}.}

\item{\code{fit_reg_df}}{Degrees of freedom of fitted regularized (lasso) models from 
\code{\link{fitRegMultiple}}.}

\item{\code{fit_null_dev}}{Deviance of fitted null models from 
\code{\link{fitNullMultiple}}.}

\item{\code{fit_null_df}}{Degrees of freedom of fitted null models from 
\code{\link{fitNullMultiple}}.}

\item{\code{fit_full_dev}}{Deviance of fitted full models from 
\code{\link{fitFullMultiple}}.}

\item{\code{fit_full_df}}{Degrees of freedom of fitted full models from 
\code{\link{fitFullMultiple}}.}

\item{\code{p_vals}}{Raw p-values (numeric vector).}

\item{\code{p_adj}}{Multiple testing adjusted p-values (Benjamini-Hochberg false discovery 
rates, FDR).}

\item{\code{LR_stats}}{Likelihood ratio test statistics.}

\item{\code{df_tests}}{Degrees of freedom of likelihood ratio tests.}
}}

\section{Accessor functions}{


\itemize{
\item \code{gene_IDs()}: Accesses gene identifiers or names.
\item \code{p_vals()}: Accesses raw p-values.
\item \code{p_adj()}: Accesses multiple testing adjusted p-values (Benjamini-Hochberg false discovery rates, FDR).
\item \code{LR_stats()}: Accesses likelihood ratio test statistics.
\item \code{df_tests()}: Accesses degrees of freedom of likelihood ratio tests.
}
}

\examples{
# initialize RegspliceResults object
gene_IDs <- paste0("gene", 1:5)
RegspliceResults(gene_IDs)

}
\seealso{
\code{\link{fitRegMultiple}} \code{\link{fitNullMultiple}}
  \code{\link{fitFullMultiple}} \code{\link{LRTests}} \code{\link{summaryTable}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/LRTests.R
\name{LRTests}
\alias{LRTests}
\title{Calculate likelihood ratio tests.}
\usage{
LRTests(rs_results, when_null_selected = c("ones", "full", "NA"))
}
\arguments{
\item{rs_results}{\code{\linkS4class{RegspliceResults}} object containing results 
generated by \code{\link{fitRegMultiple}}, \code{\link{fitNullMultiple}}, and 
\code{\link{fitFullMultiple}}. If \code{when_null_selected = "ones" or "NA"}, the 
"full" models are not required. See \code{\linkS4class{RegspliceResults}} for 
details.}

\item{when_null_selected}{Which option to use for genes where the lasso model selects 
zero interaction terms, i.e. identical to the null model. Options are \code{"ones"},
\code{"full"}, and \code{"NA"}. Default is \code{"ones"}. See below for details.}
}
\value{
Returns a \code{\linkS4class{RegspliceResults}} object containing results of 
  the LR tests. The results consist of the following entries for each gene:
\itemize{
\item p_vals: raw p-values
\item p_adj: multiple testing adjusted p-values (Benjamini-Hochberg false discovery 
rates, FDR)
\item LR_stats: likelihood ratio test statistics
\item df_tests: degrees of freedom of likelihood ratio tests
}
}
\description{
Calculate likelihood ratio tests between fitted models and null models.
}
\details{
The regularized (lasso) fitted models contain an optimal subset of exon:condition 
interaction terms for each gene, and the "full" fitted models contain all 
exon:condition interaction terms. The null models contain zero interaction terms, so 
they are nested within the fitted models.

The likelihood ratio (LR) tests compare the fitted models against the nested null 
models.

If the regularized (lasso) model contains at least one exon:condition interaction 
term, the LR test compares the lasso model against the null model. However, if the 
lasso model contains zero interaction terms, then the lasso and null models are 
identical, so the LR test cannot be calculated. The \code{when_null_selected} argument
lets the user choose what to do in these cases: either set p-values equal to 1 
(\code{when_null_selected = "ones"}); or calculate a LR test using the "full" model 
containing all exon:condition interaction terms (\code{when_null_selected = "full"}), 
which reduces power due to the larger number of terms, but allows the evidence for 
differential exon usage among these genes to be distinguished. You can also return 
\code{NA}s for these genes (\code{when_null_selected = "NA"}).

The default option is \code{when_null_selected = "ones"}. This simply calls all these 
genes non-significant, which in most cases is sufficient since we are more interested 
in genes with strong evidence for differential exon usage. However, if it is important
to rank the low-evidence genes in your data set, use the \code{when_null_selected = 
"full"} option.

If \code{when_null_selected = "ones"} or \code{when_null_selected = "NA"}, the "full" 
fitted models are not required.

Previous step: Fit models with \code{\link{fitRegMultiple}}, 
\code{\link{fitNullMultiple}}, and \code{\link{fitFullMultiple}}.
Next step: Generate summary table of results with \code{\link{summaryTable}}.
}
\examples{
file_counts <- system.file("extdata/vignette_counts.txt", package = "regsplice")
data <- read.table(file_counts, header = TRUE, sep = "\t", stringsAsFactors = FALSE)
head(data)

counts <- data[, 2:7]
tbl_exons <- table(sapply(strsplit(data$exon, ":"), function(s) s[[1]]))
gene_IDs <- names(tbl_exons)
n_exons <- unname(tbl_exons)
condition <- rep(c("untreated", "treated"), each = 3)

rs_data <- RegspliceData(counts, gene_IDs, n_exons, condition)

rs_data <- filterZeros(rs_data)
rs_data <- filterLowCounts(rs_data)
rs_data <- runNormalization(rs_data)
rs_data <- runVoom(rs_data)

rs_results <- initializeResults(rs_data)

rs_results <- fitRegMultiple(rs_results, rs_data)
rs_results <- fitNullMultiple(rs_results, rs_data)
rs_results <- fitFullMultiple(rs_results, rs_data)

rs_results <- LRTests(rs_results)

}
\seealso{
\code{\link{RegspliceResults}} \code{\link{initializeResults}} 
  \code{\link{fitRegMultiple}} \code{\link{fitNullMultiple}} 
  \code{\link{fitFullMultiple}} \code{\link{summaryTable}}
}
